"""SMTP Users API resource."""

from .base import BaseResource
from ..models.base import APIResponse
from ..models.smtp_users import (
    SmtpUsersListRequest,
    SmtpUserGetRequest,
    SmtpUserCreateRequest,
    SmtpUserUpdateRequest,
    SmtpUserDeleteRequest,
)


class SmtpUsers(BaseResource):
    """SMTP Users API resource."""

    def list_smtp_users(self, request: SmtpUsersListRequest) -> APIResponse:
        """List SMTP users for a domain.

        Args:
            request: The list SMTP users request

        Returns:
            APIResponse: API response with SMTP users list data
        """
        self.logger.debug(
            "Listing SMTP users for domain: %s with limit: %s",
            request.domain_id,
            request.query_params.limit,
        )

        # Extract query parameters
        params = request.to_query_params()

        # Make API call
        response = self.client.request(
            method="GET",
            path=f"domains/{request.domain_id}/smtp-users",
            params=params,
        )

        # Create standardized response
        return self._create_response(response)

    def get_smtp_user(self, request: SmtpUserGetRequest) -> APIResponse:
        """Get a single SMTP user.

        Args:
            request: The get SMTP user request

        Returns:
            APIResponse: API response with SMTP user data
        """
        self.logger.debug(
            "Getting SMTP user: %s from domain: %s",
            request.smtp_user_id,
            request.domain_id,
        )

        # Make API call
        response = self.client.request(
            method="GET",
            path=f"domains/{request.domain_id}/smtp-users/{request.smtp_user_id}",
        )

        # Create standardized response
        return self._create_response(response)

    def create_smtp_user(self, request: SmtpUserCreateRequest) -> APIResponse:
        """Create an SMTP user.

        Args:
            request: The create SMTP user request

        Returns:
            APIResponse: API response with SMTP user creation data
        """
        self.logger.debug(
            "Creating SMTP user: %s for domain: %s", request.name, request.domain_id
        )

        # Make API call
        response = self.client.request(
            method="POST",
            path=f"domains/{request.domain_id}/smtp-users",
            body=request.to_json(),
        )

        # Create standardized response
        return self._create_response(response)

    def update_smtp_user(self, request: SmtpUserUpdateRequest) -> APIResponse:
        """Update an SMTP user.

        Args:
            request: The update SMTP user request

        Returns:
            APIResponse: API response with updated SMTP user data
        """
        self.logger.debug(
            "Updating SMTP user: %s in domain: %s",
            request.smtp_user_id,
            request.domain_id,
        )

        # Make API call
        response = self.client.request(
            method="PUT",
            path=f"domains/{request.domain_id}/smtp-users/{request.smtp_user_id}",
            body=request.to_json(),
        )

        # Create standardized response
        return self._create_response(response)

    def delete_smtp_user(self, request: SmtpUserDeleteRequest) -> APIResponse:
        """Delete an SMTP user.

        Args:
            request: The delete SMTP user request

        Returns:
            APIResponse: API response with delete confirmation
        """
        self.logger.debug(
            "Deleting SMTP user: %s from domain: %s",
            request.smtp_user_id,
            request.domain_id,
        )

        # Make API call
        response = self.client.request(
            method="DELETE",
            path=f"domains/{request.domain_id}/smtp-users/{request.smtp_user_id}",
        )

        # Create standardized response
        return self._create_response(response)



from typing_extensions import Self
from typing import List, Dict, Optional, Any
from pydantic import (
    BaseModel,
    Field,
    EmailStr,
    ConfigDict,
    field_validator,
    model_validator,
)
import time


class EmailContact(BaseModel):
    email: EmailStr
    name: Optional[str] = None


class EmailAttachment(BaseModel):
    content: str  # Base64 encoded content
    disposition: str  # 'inline' or 'attachment'
    filename: str
    id: Optional[str] = None

    @field_validator("disposition")
    def validate_disposition(cls, v):
        if v not in ["inline", "attachment"]:
            raise ValueError("Disposition must be 'inline' or 'attachment'")
        return v


class EmailPersonalization(BaseModel):
    email: EmailStr
    data: Dict[str, Any]


class EmailTrackingSettings(BaseModel):
    track_clicks: Optional[bool] = None
    track_opens: Optional[bool] = None
    track_content: Optional[bool] = None


class EmailHeader(BaseModel):
    name: str
    value: str

    @field_validator("name")
    def validate_name(cls, v):
        if not v.replace("-", "").isalnum():
            raise ValueError("Header name must be alphanumeric and may contain '-'")
        return v


class EmailRequest(BaseModel):
    from_email: Optional[EmailContact] = Field(None, alias="from")
    to: List[EmailContact]
    cc: Optional[List[EmailContact]] = None
    bcc: Optional[List[EmailContact]] = None
    reply_to: Optional[EmailContact] = None
    subject: str
    text: Optional[str] = None
    html: Optional[str] = None
    template_id: Optional[str] = None
    attachments: Optional[List[EmailAttachment]] = None
    tags: Optional[List[str]] = None
    personalization: Optional[List[EmailPersonalization]] = None
    precedence_bulk: Optional[bool] = None
    send_at: Optional[int] = None
    in_reply_to: Optional[EmailStr] = None
    references: Optional[List[str]] = None
    settings: Optional[EmailTrackingSettings] = None
    headers: Optional[List[EmailHeader]] = None

    model_config = ConfigDict(validate_by_name=True)

    @model_validator(mode="after")
    def validate_from_email(cls, v):
        if v.from_email is None and v.template_id is None:
            raise ValueError(
                "At least one of 'from_email' or 'template_id' is required"
            )
        return v

    @field_validator("subject")
    def validate_subject_length(cls, v):
        if v and len(v) > 998:
            raise ValueError("Subject must be less than 998 characters")
        return v

    @model_validator(mode="after")
    def validate_content_exists(cls, v):
        if v.html is None and v.text is None and v.template_id is None:
            raise ValueError(
                "At least one of 'text', 'html' or 'template_id' must be provided"
            )
        return v

    @field_validator("tags")
    def validate_tags_count(cls, v):
        if v and len(v) > 5:
            raise ValueError("Maximum 5 tags are allowed")
        return v

    @field_validator("to")
    def validate_to_count(cls, v):
        if len(v) < 1 or len(v) > 50:
            raise ValueError("'to' must contain between 1 and 50 recipients")
        return v

    @field_validator("cc", "bcc")
    def validate_cc_bcc_count(cls, v):
        if v and len(v) > 10:
            raise ValueError("Maximum 10 recipients allowed for cc/bcc")
        return v

    @field_validator("send_at")
    def validate_send_at(cls, v):
        current_time = int(time.time())
        if v and (v < current_time or v > current_time + 259200):
            raise ValueError("send_at must be between now and 72 hours from now")
        return v



from typing import List, Optional, Any
from pydantic import field_validator, Field

from .base import BaseModel as MailerSendBaseModel


class MessagesListQueryParams(MailerSendBaseModel):
    """Model for messages list query parameters with validation."""

    page: Optional[int] = Field(default=1, ge=1)
    limit: Optional[int] = Field(default=25, ge=10, le=100)

    def to_query_params(self) -> dict:
        """Convert to query parameters for API request."""
        params = {"page": self.page, "limit": self.limit}

        return {k: v for k, v in params.items() if v is not None}


class MessagesListRequest(MailerSendBaseModel):
    """Request model for listing messages."""

    query_params: MessagesListQueryParams

    def to_query_params(self) -> dict:
        """Convert query parameters for API request."""
        return self.query_params.to_query_params()


class MessageGetRequest(MailerSendBaseModel):
    """Request model for getting a single message."""

    message_id: str

    @field_validator("message_id")
    def validate_message_id(cls, v):
        """Validate message ID is provided and not empty."""
        if not v or not v.strip():
            raise ValueError("Message ID is required")
        return v.strip()



import platform
import sys

# Base API information
API_VERSION = "v1"
DEFAULT_BASE_URL = f"https://api.mailersend.com/{API_VERSION}/"
DEFAULT_TIMEOUT = 30  # seconds

# Package info for user agent
PACKAGE_NAME = "mailersend-python"
__version__ = "2.0.0"

USER_AGENT = (
    f"{PACKAGE_NAME}/{__version__} "
    f"(Python/{platform.python_version()}; "
    f"OS/{platform.system()} {platform.release()}; "
    f"Impl/{platform.python_implementation()})"
)


"""
SMS Activity API models.
"""

from datetime import datetime
from typing import Optional, List
from pydantic import Field

from .base import BaseModel as MailerSendBaseModel


class SmsActivity(MailerSendBaseModel):
    """SMS Activity model."""

    from_: str = Field(..., alias="from")
    to: str
    created_at: datetime
    content: str
    status: str
    sms_message_id: str


class SmsActivityListRequest(MailerSendBaseModel):
    """Request model for listing SMS activities."""

    sms_number_id: Optional[str] = None
    date_from: Optional[int] = None
    date_to: Optional[int] = None
    status: Optional[List[str]] = None
    page: Optional[int] = None
    limit: Optional[int] = None

    def to_query_params(self) -> dict:
        """Convert to query parameters."""
        params = {}

        if self.sms_number_id is not None:
            params["sms_number_id"] = self.sms_number_id

        if self.date_from is not None:
            params["date_from"] = self.date_from

        if self.date_to is not None:
            params["date_to"] = self.date_to

        if self.status is not None:
            # Convert list to multiple status[] parameters
            for status_value in self.status:
                if "status[]" not in params:
                    params["status[]"] = []
                params["status[]"].append(status_value)

        if self.page is not None:
            params["page"] = self.page

        if self.limit is not None:
            params["limit"] = self.limit

        return params


class SmsMessageGetRequest(MailerSendBaseModel):
    """Request model for getting SMS message activity."""

    sms_message_id: str = Field(..., min_length=1)


class SmsMessage(MailerSendBaseModel):
    """SMS Message model."""

    id: str
    from_: str = Field(..., alias="from")
    to: List[str]
    text: str
    paused: bool
    created_at: datetime
    sms: List[dict]
    sms_activity: List[SmsActivity]



"""Email Verification API models for MailerSend SDK."""

from datetime import datetime
from typing import List, Optional, Dict, Any

from pydantic import BaseModel, Field, field_validator


# Query Parameters Models
class EmailVerificationListsQueryParams(BaseModel):
    """Query parameters for listing email verification lists."""

    page: int = Field(default=1, ge=1, description="Page number")
    limit: int = Field(default=25, ge=10, le=100, description="Items per page")

    def to_query_params(self) -> Dict[str, Any]:
        """Convert to query parameters dictionary."""
        params = {}
        if self.page is not None:
            params["page"] = self.page
        if self.limit is not None:
            params["limit"] = self.limit
        return params


class EmailVerificationResultsQueryParams(BaseModel):
    """Query parameters for getting email verification results."""

    page: int = Field(default=1, ge=1, description="Page number")
    limit: int = Field(default=25, ge=10, le=100, description="Items per page")
    results: Optional[List[str]] = Field(
        None, description="Filter by specific verification results"
    )

    @field_validator("results")
    @classmethod
    def validate_results(cls, v: Optional[List[str]]) -> Optional[List[str]]:
        """Validate results filter values."""
        if v is not None:
            valid_results = {
                "valid",
                "catch_all",
                "mailbox_full",
                "role_based",
                "unknown",
                "failed",
                "syntax_error",
                "typo",
                "mailbox_not_found",
                "disposable",
                "mailbox_blocked",
            }
            for result in v:
                if result not in valid_results:
                    raise ValueError(f"Invalid result filter: {result}")
        return v

    def to_query_params(self) -> Dict[str, Any]:
        """Convert to query parameters dictionary."""
        params = {}
        if self.page is not None:
            params["page"] = self.page
        if self.limit is not None:
            params["limit"] = self.limit
        if self.results is not None and self.results:
            params["results"] = self.results
        return params


# Request Models
class EmailVerifyRequest(BaseModel):
    """Request model for single email verification."""

    email: str = Field(..., description="Email address to verify")

    @field_validator("email")
    @classmethod
    def validate_email(cls, v: str) -> str:
        """Validate email is not empty."""
        if not v or not v.strip():
            raise ValueError("email cannot be empty")
        return v.strip()


class EmailVerifyAsyncRequest(BaseModel):
    """Request model for async single email verification."""

    email: str = Field(..., description="Email address to verify asynchronously")

    @field_validator("email")
    @classmethod
    def validate_email(cls, v: str) -> str:
        """Validate email is not empty."""
        if not v or not v.strip():
            raise ValueError("email cannot be empty")
        return v.strip()


class EmailVerificationAsyncStatusRequest(BaseModel):
    """Request model for getting async email verification status."""

    email_verification_id: str = Field(..., description="Email verification ID")

    @field_validator("email_verification_id")
    @classmethod
    def validate_email_verification_id(cls, v: str) -> str:
        """Validate email_verification_id is not empty."""
        if not v or not v.strip():
            raise ValueError("email_verification_id cannot be empty")
        return v.strip()


class EmailVerificationListsRequest(BaseModel):
    """Request model for listing email verification lists."""

    query_params: EmailVerificationListsQueryParams

    def to_query_params(self) -> Dict[str, Any]:
        """Convert to query parameters dictionary."""
        return self.query_params.to_query_params()


class EmailVerificationGetRequest(BaseModel):
    """Request model for getting a single email verification list."""

    email_verification_id: str = Field(..., description="Email verification ID")

    @field_validator("email_verification_id")
    @classmethod
    def validate_email_verification_id(cls, v: str) -> str:
        """Validate email_verification_id is not empty."""
        if not v or not v.strip():
            raise ValueError("email_verification_id cannot be empty")
        return v.strip()


class EmailVerificationCreateRequest(BaseModel):
    """Request model for creating an email verification list."""

    name: str = Field(..., description="Name of the verification list")
    emails: List[str] = Field(
        ..., min_length=1, description="List of email addresses to verify"
    )

    @field_validator("name")
    @classmethod
    def validate_name(cls, v: str) -> str:
        """Validate name is not empty."""
        if not v or not v.strip():
            raise ValueError("name cannot be empty")
        return v.strip()

    @field_validator("emails")
    @classmethod
    def validate_emails(cls, v: List[str]) -> List[str]:
        """Validate emails list is not empty and all emails are valid."""
        if not v:
            raise ValueError("emails list cannot be empty")

        validated_emails = []
        for email in v:
            if not email or not email.strip():
                raise ValueError("email addresses cannot be empty")
            if len(email.strip()) > 191:
                raise ValueError("email addresses cannot exceed 191 characters")
            validated_emails.append(email.strip())

        return validated_emails


class EmailVerificationVerifyRequest(BaseModel):
    """Request model for verifying an email verification list."""

    email_verification_id: str = Field(..., description="Email verification ID")

    @field_validator("email_verification_id")
    @classmethod
    def validate_email_verification_id(cls, v: str) -> str:
        """Validate email_verification_id is not empty."""
        if not v or not v.strip():
            raise ValueError("email_verification_id cannot be empty")
        return v.strip()


class EmailVerificationResultsRequest(BaseModel):
    """Request model for getting email verification results."""

    email_verification_id: str = Field(..., description="Email verification ID")
    query_params: EmailVerificationResultsQueryParams

    @field_validator("email_verification_id")
    @classmethod
    def validate_email_verification_id(cls, v: str) -> str:
        """Validate email_verification_id is not empty."""
        if not v or not v.strip():
            raise ValueError("email_verification_id cannot be empty")
        return v.strip()

    def to_query_params(self) -> Dict[str, Any]:
        """Convert to query parameters dictionary."""
        return self.query_params.to_query_params()



"""
Builder patterns for constructing complex MailerSend requests.

The builder pattern provides a fluent, chainable API for constructing
complex email requests with intelligent defaults and validation.
"""

from .email import EmailBuilder
from .activity import ActivityBuilder, SingleActivityBuilder
from .analytics import AnalyticsBuilder
from .domains import DomainsBuilder
from .identities import IdentityBuilder
from .inbound import InboundBuilder
from .messages import MessagesBuilder
from .schedules import SchedulesBuilder
from .recipients import RecipientsBuilder
from .templates import TemplatesBuilder
from .tokens import TokensBuilder
from .webhooks import WebhooksBuilder
from .email_verification import EmailVerificationBuilder
from .users import UsersBuilder
from .sms_messages import SmsMessagesBuilder
from .sms_numbers import SmsNumbersBuilder
from .sms_activity import SmsActivityBuilder
from .sms_sending import SmsSendingBuilder
from .sms_recipients import SmsRecipientsBuilder
from .sms_webhooks import SmsWebhooksBuilder
from .sms_inbounds import SmsInboundsBuilder

__all__ = [
    "EmailBuilder",
    "ActivityBuilder",
    "SingleActivityBuilder",
    "AnalyticsBuilder",
    "DomainsBuilder",
    "IdentityBuilder",
    "InboundBuilder",
    "MessagesBuilder",
    "SchedulesBuilder",
    "RecipientsBuilder",
    "TemplatesBuilder",
    "TokensBuilder",
    "WebhooksBuilder",
    "EmailVerificationBuilder",
    "UsersBuilder",
    "SmsMessagesBuilder",
    "SmsNumbersBuilder",
    "SmsActivityBuilder",
    "SmsSendingBuilder",
    "SmsRecipientsBuilder",
    "SmsWebhooksBuilder",
    "SmsInboundsBuilder",
]



from typing import Optional, Union
from copy import deepcopy

from ..models.domains import (
    DomainListRequest,
    DomainListQueryParams,
    DomainCreateRequest,
    DomainDeleteRequest,
    DomainGetRequest,
    DomainUpdateSettingsRequest,
    DomainRecipientsRequest,
    DomainRecipientsQueryParams,
    DomainDnsRecordsRequest,
    DomainVerificationRequest,
)
from ..exceptions import ValidationError


class DomainsBuilder:
    """
    Builder for creating domain-related requests using a fluent interface.

    Supports building requests for:
    - Listing domains
    - Creating domains
    - Getting single domain
    - Deleting domains
    - Updating domain settings
    - Getting domain recipients
    - Getting domain DNS records
    - Getting domain verification status
    """

    def __init__(self):
        """Initialize a new DomainsBuilder."""
        self._reset()

    def _reset(self):
        """Reset all builder state."""
        # List domains parameters
        self._page: Optional[int] = None
        self._limit: Optional[int] = None
        self._verified: Optional[bool] = None

        # Create domain parameters
        self._name: Optional[str] = None
        self._return_path_subdomain: Optional[str] = None
        self._custom_tracking_subdomain: Optional[str] = None
        self._inbound_routing_subdomain: Optional[str] = None

        # Delete domain parameters
        self._domain_id: Optional[str] = None

        # Update domain settings parameters
        self._send_paused: Optional[bool] = None
        self._track_clicks: Optional[bool] = None
        self._track_opens: Optional[bool] = None
        self._track_unsubscribe: Optional[bool] = None
        self._track_content: Optional[bool] = None
        self._track_unsubscribe_html: Optional[str] = None
        self._track_unsubscribe_plain: Optional[str] = None
        self._custom_tracking_enabled: Optional[bool] = None
        self._custom_tracking_subdomain_setting: Optional[str] = None
        self._precedence_bulk: Optional[bool] = None
        self._ignore_duplicated_recipients: Optional[bool] = None

    # Pagination methods (used by list and recipients)
    def page(self, page: int) -> "DomainsBuilder":
        """
        Set the page number for pagination.

        Args:
            page: Page number (must be > 0)

        Returns:
            Self for method chaining
        """
        if page < 1:
            raise ValidationError("Page must be greater than 0")
        self._page = page
        return self

    def limit(self, limit: int) -> "DomainsBuilder":
        """
        Set the number of items per page.

        Args:
            limit: Items per page (10-100)

        Returns:
            Self for method chaining
        """
        if limit < 10 or limit > 100:
            raise ValidationError("Limit must be between 10 and 100")
        self._limit = limit
        return self

    # Domain list specific methods
    def verified(self, verified: bool) -> "DomainsBuilder":
        """
        Filter domains by verification status.

        Args:
            verified: True for verified domains only, False for unverified

        Returns:
            Self for method chaining
        """
        self._verified = verified
        return self

    def verified_only(self) -> "DomainsBuilder":
        """
        Show only verified domains.

        Returns:
            Self for method chaining
        """
        return self.verified(True)

    def unverified_only(self) -> "DomainsBuilder":
        """
        Show only unverified domains.

        Returns:
            Self for method chaining
        """
        return self.verified(False)

    # Domain creation methods
    def domain_name(self, name: str) -> "DomainsBuilder":
        """
        Set the domain name for creation.

        Args:
            name: Domain name (must be lowercase and valid format)

        Returns:
            Self for method chaining
        """
        self._name = name
        return self

    def return_path_subdomain(self, subdomain: str) -> "DomainsBuilder":
        """
        Set the return path subdomain.

        Args:
            subdomain: Return path subdomain (must be alphanumeric)

        Returns:
            Self for method chaining
        """
        self._return_path_subdomain = subdomain
        return self

    def custom_tracking_subdomain(self, subdomain: str) -> "DomainsBuilder":
        """
        Set the custom tracking subdomain.

        Args:
            subdomain: Custom tracking subdomain (must be alphanumeric)

        Returns:
            Self for method chaining
        """
        self._custom_tracking_subdomain = subdomain
        return self

    def inbound_routing_subdomain(self, subdomain: str) -> "DomainsBuilder":
        """
        Set the inbound routing subdomain.

        Args:
            subdomain: Inbound routing subdomain (must be alphanumeric)

        Returns:
            Self for method chaining
        """
        self._inbound_routing_subdomain = subdomain
        return self

    # Domain deletion methods
    def domain_id(self, domain_id: str) -> "DomainsBuilder":
        """
        Set the domain ID for deletion.

        Args:
            domain_id: Domain ID to delete

        Returns:
            Self for method chaining
        """
        self._domain_id = domain_id
        return self

    # Domain settings update methods
    def send_paused(self, paused: bool) -> "DomainsBuilder":
        """
        Set the send paused status.

        Args:
            paused: True to pause sending, False to resume

        Returns:
            Self for method chaining
        """
        self._send_paused = paused
        return self

    def pause_sending(self) -> "DomainsBuilder":
        """
        Pause email sending for the domain.

        Returns:
            Self for method chaining
        """
        return self.send_paused(True)

    def resume_sending(self) -> "DomainsBuilder":
        """
        Resume email sending for the domain.

        Returns:
            Self for method chaining
        """
        return self.send_paused(False)

    def track_clicks(self, enabled: bool) -> "DomainsBuilder":
        """
        Set click tracking status.

        Args:
            enabled: True to enable click tracking, False to disable

        Returns:
            Self for method chaining
        """
        self._track_clicks = enabled
        return self

    def track_opens(self, enabled: bool) -> "DomainsBuilder":
        """
        Set open tracking status.

        Args:
            enabled: True to enable open tracking, False to disable

        Returns:
            Self for method chaining
        """
        self._track_opens = enabled
        return self

    def track_unsubscribe(self, enabled: bool) -> "DomainsBuilder":
        """
        Set unsubscribe tracking status.

        Args:
            enabled: True to enable unsubscribe tracking, False to disable

        Returns:
            Self for method chaining
        """
        self._track_unsubscribe = enabled
        return self

    def track_content(self, enabled: bool) -> "DomainsBuilder":
        """
        Set content tracking status.

        Args:
            enabled: True to enable content tracking, False to disable

        Returns:
            Self for method chaining
        """
        self._track_content = enabled
        return self

    def track_unsubscribe_html(self, html: str) -> "DomainsBuilder":
        """
        Set HTML unsubscribe tracking content.

        Args:
            html: HTML content for unsubscribe tracking

        Returns:
            Self for method chaining
        """
        self._track_unsubscribe_html = html
        return self

    def track_unsubscribe_plain(self, text: str) -> "DomainsBuilder":
        """
        Set plain text unsubscribe tracking content.

        Args:
            text: Plain text content for unsubscribe tracking

        Returns:
            Self for method chaining
        """
        self._track_unsubscribe_plain = text
        return self

    def custom_tracking_enabled(self, enabled: bool) -> "DomainsBuilder":
        """
        Set custom tracking status.

        Args:
            enabled: True to enable custom tracking, False to disable

        Returns:
            Self for method chaining
        """
        self._custom_tracking_enabled = enabled
        return self

    def custom_tracking_subdomain_setting(self, subdomain: str) -> "DomainsBuilder":
        """
        Set custom tracking subdomain.

        Args:
            subdomain: Custom tracking subdomain

        Returns:
            Self for method chaining
        """
        self._custom_tracking_subdomain_setting = subdomain
        return self

    def precedence_bulk(self, enabled: bool) -> "DomainsBuilder":
        """
        Set bulk precedence status.

        Args:
            enabled: True to enable bulk precedence, False to disable

        Returns:
            Self for method chaining
        """
        self._precedence_bulk = enabled
        return self

    def ignore_duplicated_recipients(self, enabled: bool) -> "DomainsBuilder":
        """
        Set ignore duplicated recipients status.

        Args:
            enabled: True to ignore duplicated recipients, False to include

        Returns:
            Self for method chaining
        """
        self._ignore_duplicated_recipients = enabled
        return self

    def enable_all_tracking(self) -> "DomainsBuilder":
        """
        Enable all tracking features.

        Returns:
            Self for method chaining
        """
        return (
            self.track_clicks(True)
            .track_opens(True)
            .track_unsubscribe(True)
            .track_content(True)
        )

    def disable_all_tracking(self) -> "DomainsBuilder":
        """
        Disable all tracking features.

        Returns:
            Self for method chaining
        """
        return (
            self.track_clicks(False)
            .track_opens(False)
            .track_unsubscribe(False)
            .track_content(False)
        )

    # Build methods for different request types
    def build_list_request(self) -> DomainListRequest:
        """
        Build a domain list request.

        Returns:
            DomainListRequest object
        """
        # Create query params with proper defaults, using builder values only if set
        query_params_data = {}
        if self._page is not None:
            query_params_data["page"] = self._page
        if self._limit is not None:
            query_params_data["limit"] = self._limit
        if self._verified is not None:
            query_params_data["verified"] = self._verified

        query_params = DomainListQueryParams(**query_params_data)
        return DomainListRequest(query_params=query_params)

    def build_create_request(self) -> DomainCreateRequest:
        """
        Build a domain creation request.

        Returns:
            DomainCreateRequest object

        Raises:
            ValidationError: If domain name is not provided
        """
        if not self._name:
            raise ValidationError("Domain name is required for creation")

        return DomainCreateRequest(
            name=self._name,
            return_path_subdomain=self._return_path_subdomain,
            custom_tracking_subdomain=self._custom_tracking_subdomain,
            inbound_routing_subdomain=self._inbound_routing_subdomain,
        )

    def build_delete_request(self) -> DomainDeleteRequest:
        """
        Build a domain deletion request.

        Returns:
            DomainDeleteRequest object

        Raises:
            ValidationError: If domain ID is not provided
        """
        if not self._domain_id:
            raise ValidationError("Domain ID is required for deletion")

        return DomainDeleteRequest(domain_id=self._domain_id)

    def build_update_settings_request(self) -> DomainUpdateSettingsRequest:
        """
        Build a domain settings update request.

        Returns:
            DomainUpdateSettingsRequest object

        Raises:
            ValidationError: If domain ID is not provided
        """
        if not self._domain_id:
            raise ValidationError("Domain ID is required for updating settings")

        return DomainUpdateSettingsRequest(
            domain_id=self._domain_id,
            send_paused=self._send_paused,
            track_clicks=self._track_clicks,
            track_opens=self._track_opens,
            track_unsubscribe=self._track_unsubscribe,
            track_content=self._track_content,
            track_unsubscribe_html=self._track_unsubscribe_html,
            track_unsubscribe_plain=self._track_unsubscribe_plain,
            custom_tracking_enabled=self._custom_tracking_enabled,
            custom_tracking_subdomain=self._custom_tracking_subdomain_setting,
            precedence_bulk=self._precedence_bulk,
            ignore_duplicated_recipients=self._ignore_duplicated_recipients,
        )

    def build_recipients_request(self) -> DomainRecipientsRequest:
        """
        Build a domain recipients request.

        Returns:
            DomainRecipientsRequest object

        Raises:
            ValidationError: If domain ID is not provided
        """
        if not self._domain_id:
            raise ValidationError("Domain ID is required for getting recipients")

        # Create query params with proper defaults, using builder values only if set
        query_params_data = {}
        if self._page is not None:
            query_params_data["page"] = self._page
        if self._limit is not None:
            query_params_data["limit"] = self._limit

        query_params = DomainRecipientsQueryParams(**query_params_data)
        return DomainRecipientsRequest(
            domain_id=self._domain_id, query_params=query_params
        )

    def build_get_request(self) -> DomainGetRequest:
        """
        Build a domain get request.

        Returns:
            DomainGetRequest object

        Raises:
            ValidationError: If domain ID is not provided
        """
        if not self._domain_id:
            raise ValidationError("Domain ID is required for getting domain")

        return DomainGetRequest(domain_id=self._domain_id)

    def build_dns_records_request(self) -> DomainDnsRecordsRequest:
        """
        Build a domain DNS records request.

        Returns:
            DomainDnsRecordsRequest object

        Raises:
            ValidationError: If domain ID is not provided
        """
        if not self._domain_id:
            raise ValidationError("Domain ID is required for getting DNS records")

        return DomainDnsRecordsRequest(domain_id=self._domain_id)

    def build_verification_request(self) -> DomainVerificationRequest:
        """
        Build a domain verification request.

        Returns:
            DomainVerificationRequest object

        Raises:
            ValidationError: If domain ID is not provided
        """
        if not self._domain_id:
            raise ValidationError(
                "Domain ID is required for getting verification status"
            )

        return DomainVerificationRequest(domain_id=self._domain_id)

    # State management methods
    def reset(self) -> "DomainsBuilder":
        """
        Reset the builder to its initial state.

        Returns:
            Self for method chaining
        """
        self._reset()
        return self

    def copy(self) -> "DomainsBuilder":
        """
        Create a copy of this builder.

        Returns:
            New DomainsBuilder instance with the same state
        """
        new_builder = DomainsBuilder()
        new_builder._page = self._page
        new_builder._limit = self._limit
        new_builder._verified = self._verified
        new_builder._name = self._name
        new_builder._return_path_subdomain = self._return_path_subdomain
        new_builder._custom_tracking_subdomain = self._custom_tracking_subdomain
        new_builder._inbound_routing_subdomain = self._inbound_routing_subdomain
        new_builder._domain_id = self._domain_id
        new_builder._send_paused = self._send_paused
        new_builder._track_clicks = self._track_clicks
        new_builder._track_opens = self._track_opens
        new_builder._track_unsubscribe = self._track_unsubscribe
        new_builder._track_content = self._track_content
        new_builder._track_unsubscribe_html = self._track_unsubscribe_html
        new_builder._track_unsubscribe_plain = self._track_unsubscribe_plain
        new_builder._custom_tracking_enabled = self._custom_tracking_enabled
        new_builder._custom_tracking_subdomain_setting = (
            self._custom_tracking_subdomain_setting
        )
        new_builder._precedence_bulk = self._precedence_bulk
        new_builder._ignore_duplicated_recipients = self._ignore_duplicated_recipients
        return new_builder



"""
API resource classes for interacting with specific MailerSend API endpoints.
"""

from .base import BaseResource
from .email import Email
from .activity import Activity
from .analytics import Analytics
from .domains import Domains
from .identities import IdentitiesResource
from .inbound import InboundResource
from .messages import Messages
from .schedules import Schedules
from .recipients import Recipients
from .templates import Templates
from .tokens import Tokens
from .webhooks import Webhooks
from .email_verification import EmailVerification
from .users import Users
from .sms_messages import SmsMessages
from .sms_numbers import SmsNumbers
from .sms_activity import SmsActivity
from .sms_sending import SmsSending
from .sms_recipients import SmsRecipients
from .sms_webhooks import SmsWebhooks
from .sms_inbounds import SmsInbounds
from .other import Other

__all__ = [
    "BaseResource",
    "Email",
    "Activity",
    "Analytics",
    "Domains",
    "IdentitiesResource",
    "InboundResource",
    "Messages",
    "Schedules",
    "Recipients",
    "Templates",
    "Tokens",
    "Webhooks",
    "EmailVerification",
    "Users",
    "SmsMessages",
    "SmsNumbers",
    "SmsActivity",
    "SmsSending",
    "SmsRecipients",
    "SmsWebhooks",
    "SmsInbounds",
    "Other",
]



"""
Email Builder - Fluent API for constructing complex email requests.

This module provides the EmailBuilder class which offers a chainable,
developer-friendly API for building complex email requests with
intelligent defaults, file handling, and validation.
"""

import base64
import mimetypes
from datetime import datetime, timezone
from pathlib import Path
from typing import List, Dict, Any, Optional, Union, IO

from ..models.email import (
    EmailRequest,
    EmailContact,
    EmailAttachment,
    EmailPersonalization,
    EmailTrackingSettings,
    EmailHeader,
)
from ..exceptions import ValidationError


class EmailBuilder:
    """
    Fluent builder for constructing EmailRequest objects.

    Provides a chainable API that makes it easy to build complex emails
    with attachments, personalization, tracking settings, and more.

    Examples:
        >>> # Simple email
        >>> email = (EmailBuilder()
        ...     .from_email("sender@example.com", "Sender Name")
        ...     .to("recipient@example.com")
        ...     .subject("Hello World")
        ...     .html("<h1>Hello!</h1>")
        ...     .build())

        >>> # Complex email with all features
        >>> email = (EmailBuilder()
        ...     .from_email("sender@example.com", "Marketing Team")
        ...     .to("user1@example.com", "John Doe")
        ...     .to("user2@example.com", "Jane Smith")
        ...     .cc("manager@example.com")
        ...     .subject("Monthly Newsletter")
        ...     .html_file("templates/newsletter.html")
        ...     .attach_file("documents/report.pdf")
        ...     .personalize("user1@example.com", name="John", company="Acme")
        ...     .tag("newsletter", "monthly")
        ...     .track_clicks(True)
        ...     .header("X-Campaign-ID", "newsletter-2024-01")
        ...     .build())
    """

    def __init__(self):
        """Initialize a new EmailBuilder."""
        self._from_email: Optional[EmailContact] = None
        self._to: List[EmailContact] = []
        self._cc: List[EmailContact] = []
        self._bcc: List[EmailContact] = []
        self._reply_to: Optional[EmailContact] = None
        self._subject: Optional[str] = None
        self._html: Optional[str] = None
        self._text: Optional[str] = None
        self._template_id: Optional[str] = None
        self._attachments: List[EmailAttachment] = []
        self._tags: List[str] = []
        self._personalization: List[EmailPersonalization] = []
        self._precedence_bulk: Optional[bool] = None
        self._send_at: Optional[int] = None
        self._in_reply_to: Optional[str] = None
        self._references: List[str] = []
        self._settings: Optional[EmailTrackingSettings] = None
        self._headers: List[EmailHeader] = []

    def from_email(self, email: str, name: Optional[str] = None) -> "EmailBuilder":
        """
        Set the sender email address.

        Args:
            email: Sender email address
            name: Optional sender name

        Returns:
            EmailBuilder instance for chaining
        """
        self._from_email = EmailContact(email=email, name=name)
        return self

    def to(self, email: str, name: Optional[str] = None) -> "EmailBuilder":
        """
        Add a recipient to the TO field.

        Args:
            email: Recipient email address
            name: Optional recipient name

        Returns:
            EmailBuilder instance for chaining
        """
        self._to.append(EmailContact(email=email, name=name))
        return self

    def to_many(self, recipients: List[Dict[str, str]]) -> "EmailBuilder":
        """
        Add multiple recipients to the TO field.

        Args:
            recipients: List of dicts with 'email' and optional 'name' keys

        Returns:
            EmailBuilder instance for chaining

        Example:
            >>> builder.to_many([
            ...     {"email": "user1@example.com", "name": "User 1"},
            ...     {"email": "user2@example.com", "name": "User 2"}
            ... ])
        """
        for recipient in recipients:
            self._to.append(
                EmailContact(email=recipient["email"], name=recipient.get("name"))
            )
        return self

    def cc(self, email: str, name: Optional[str] = None) -> "EmailBuilder":
        """
        Add a recipient to the CC field.

        Args:
            email: CC recipient email address
            name: Optional recipient name

        Returns:
            EmailBuilder instance for chaining
        """
        self._cc.append(EmailContact(email=email, name=name))
        return self

    def bcc(self, email: str, name: Optional[str] = None) -> "EmailBuilder":
        """
        Add a recipient to the BCC field.

        Args:
            email: BCC recipient email address
            name: Optional recipient name

        Returns:
            EmailBuilder instance for chaining
        """
        self._bcc.append(EmailContact(email=email, name=name))
        return self

    def reply_to(self, email: str, name: Optional[str] = None) -> "EmailBuilder":
        """
        Set the reply-to email address.

        Args:
            email: Reply-to email address
            name: Optional reply-to name

        Returns:
            EmailBuilder instance for chaining
        """
        self._reply_to = EmailContact(email=email, name=name)
        return self

    def subject(self, subject: str) -> "EmailBuilder":
        """
        Set the email subject.

        Args:
            subject: Email subject line

        Returns:
            EmailBuilder instance for chaining
        """
        self._subject = subject
        return self

    def html(self, html_content: str) -> "EmailBuilder":
        """
        Set the HTML content of the email.

        Args:
            html_content: HTML content string

        Returns:
            EmailBuilder instance for chaining
        """
        self._html = html_content
        return self

    def html_file(self, file_path: Union[str, Path]) -> "EmailBuilder":
        """
        Load HTML content from a file.

        Args:
            file_path: Path to HTML file

        Returns:
            EmailBuilder instance for chaining

        Raises:
            ValidationError: If file cannot be read
        """
        try:
            path = Path(file_path)
            self._html = path.read_text(encoding="utf-8")
        except Exception as e:
            raise ValidationError(f"Failed to read HTML file {file_path}: {str(e)}")
        return self

    def text(self, text_content: str) -> "EmailBuilder":
        """
        Set the plain text content of the email.

        Args:
            text_content: Plain text content string

        Returns:
            EmailBuilder instance for chaining
        """
        self._text = text_content
        return self

    def text_file(self, file_path: Union[str, Path]) -> "EmailBuilder":
        """
        Load plain text content from a file.

        Args:
            file_path: Path to text file

        Returns:
            EmailBuilder instance for chaining

        Raises:
            ValidationError: If file cannot be read
        """
        try:
            path = Path(file_path)
            self._text = path.read_text(encoding="utf-8")
        except Exception as e:
            raise ValidationError(f"Failed to read text file {file_path}: {str(e)}")
        return self

    def template(self, template_id: str) -> "EmailBuilder":
        """
        Use a template for the email content.

        Args:
            template_id: MailerSend template ID

        Returns:
            EmailBuilder instance for chaining
        """
        self._template_id = template_id
        return self

    def attach_file(
        self,
        file_path: Union[str, Path],
        filename: Optional[str] = None,
        disposition: str = "attachment",
    ) -> "EmailBuilder":
        """
        Attach a file to the email.

        Args:
            file_path: Path to file to attach
            filename: Optional custom filename (defaults to actual filename)
            disposition: 'attachment' or 'inline'

        Returns:
            EmailBuilder instance for chaining

        Raises:
            ValidationError: If file cannot be read or disposition is invalid
        """
        try:
            path = Path(file_path)
            if not path.exists():
                raise ValidationError(f"File not found: {file_path}")

            # Read and encode file content
            content = base64.b64encode(path.read_bytes()).decode("utf-8")

            # Use provided filename or extract from path
            final_filename = filename or path.name

            attachment = EmailAttachment(
                content=content, filename=final_filename, disposition=disposition
            )

            self._attachments.append(attachment)

        except Exception as e:
            if isinstance(e, ValidationError):
                raise
            raise ValidationError(f"Failed to attach file {file_path}: {str(e)}")

        return self

    def attach_content(
        self, content: Union[str, bytes], filename: str, disposition: str = "attachment"
    ) -> "EmailBuilder":
        """
        Attach content directly (without reading from file).

        Args:
            content: Content to attach (string or bytes)
            filename: Filename for the attachment
            disposition: 'attachment' or 'inline'

        Returns:
            EmailBuilder instance for chaining
        """
        if isinstance(content, str):
            content = content.encode("utf-8")

        encoded_content = base64.b64encode(content).decode("utf-8")

        attachment = EmailAttachment(
            content=encoded_content, filename=filename, disposition=disposition
        )

        self._attachments.append(attachment)
        return self

    def tag(self, *tags: str) -> "EmailBuilder":
        """
        Add one or more tags to the email.

        Args:
            *tags: Tag names to add

        Returns:
            EmailBuilder instance for chaining

        Example:
            >>> builder.tag("newsletter", "marketing", "monthly")
        """
        self._tags.extend(tags)
        return self

    def personalize(self, email: str, **data: Any) -> "EmailBuilder":
        """
        Add personalization data for a specific recipient.

        Args:
            email: Recipient email address
            **data: Key-value pairs for personalization

        Returns:
            EmailBuilder instance for chaining

        Example:
            >>> builder.personalize("user@example.com", name="John", company="Acme")
        """
        personalization = EmailPersonalization(email=email, data=data)
        self._personalization.append(personalization)
        return self

    def personalize_many(
        self, personalizations: List[Dict[str, Any]]
    ) -> "EmailBuilder":
        """
        Add personalization data for multiple recipients.

        Args:
            personalizations: List of personalization objects

        Returns:
            EmailBuilder instance for chaining

        Example:
            >>> builder.personalize_many([
            ...     {"email": "user1@example.com", "data": {"name": "John"}},
            ...     {"email": "user2@example.com", "data": {"name": "Jane"}}
            ... ])
        """
        for p in personalizations:
            personalization = EmailPersonalization(email=p["email"], data=p["data"])
            self._personalization.append(personalization)
        return self

    def bulk_mode(self, enabled: bool = True) -> "EmailBuilder":
        """
        Enable or disable bulk precedence for the email.

        Args:
            enabled: Whether to enable bulk mode

        Returns:
            EmailBuilder instance for chaining
        """
        self._precedence_bulk = enabled
        return self

    def send_at(self, send_time: Union[datetime, int]) -> "EmailBuilder":
        """
        Schedule the email to be sent at a specific time.

        Args:
            send_time: When to send the email (datetime or unix timestamp)

        Returns:
            EmailBuilder instance for chaining
        """
        if isinstance(send_time, datetime):
            # Convert to UTC timestamp
            if send_time.tzinfo is None:
                send_time = send_time.replace(tzinfo=timezone.utc)
            self._send_at = int(send_time.timestamp())
        else:
            self._send_at = send_time
        return self

    def send_in(self, **kwargs) -> "EmailBuilder":
        """
        Schedule the email to be sent after a specific duration.

        Args:
            **kwargs: Duration arguments (hours, minutes, seconds, days)

        Returns:
            EmailBuilder instance for chaining

        Example:
            >>> builder.send_in(hours=2, minutes=30)  # Send in 2.5 hours
        """
        from datetime import timedelta

        now = datetime.now(timezone.utc)
        delta = timedelta(**kwargs)
        send_time = now + delta

        return self.send_at(send_time)

    def in_reply_to(self, email: str) -> "EmailBuilder":
        """
        Set the In-Reply-To email for threading.

        Args:
            email: Email address this email is replying to

        Returns:
            EmailBuilder instance for chaining
        """
        self._in_reply_to = email
        return self

    def reference(self, *message_ids: str) -> "EmailBuilder":
        """
        Add message IDs to the References header for threading.

        Args:
            *message_ids: Message IDs to reference

        Returns:
            EmailBuilder instance for chaining
        """
        self._references.extend(message_ids)
        return self

    def track_clicks(self, enabled: bool = True) -> "EmailBuilder":
        """
        Enable or disable click tracking.

        Args:
            enabled: Whether to track clicks

        Returns:
            EmailBuilder instance for chaining
        """
        if self._settings is None:
            self._settings = EmailTrackingSettings()
        self._settings.track_clicks = enabled
        return self

    def track_opens(self, enabled: bool = True) -> "EmailBuilder":
        """
        Enable or disable open tracking.

        Args:
            enabled: Whether to track opens

        Returns:
            EmailBuilder instance for chaining
        """
        if self._settings is None:
            self._settings = EmailTrackingSettings()
        self._settings.track_opens = enabled
        return self

    def track_content(self, enabled: bool = True) -> "EmailBuilder":
        """
        Enable or disable content tracking.

        Args:
            enabled: Whether to track content

        Returns:
            EmailBuilder instance for chaining
        """
        if self._settings is None:
            self._settings = EmailTrackingSettings()
        self._settings.track_content = enabled
        return self

    def tracking(
        self,
        clicks: Optional[bool] = None,
        opens: Optional[bool] = None,
        content: Optional[bool] = None,
    ) -> "EmailBuilder":
        """
        Configure all tracking settings at once.

        Args:
            clicks: Whether to track clicks
            opens: Whether to track opens
            content: Whether to track content

        Returns:
            EmailBuilder instance for chaining
        """
        self._settings = EmailTrackingSettings(
            track_clicks=clicks, track_opens=opens, track_content=content
        )
        return self

    def header(self, name: str, value: str) -> "EmailBuilder":
        """
        Add a custom header to the email.

        Args:
            name: Header name
            value: Header value

        Returns:
            EmailBuilder instance for chaining
        """
        header = EmailHeader(name=name, value=value)
        self._headers.append(header)
        return self

    def headers(self, headers_dict: Dict[str, str]) -> "EmailBuilder":
        """
        Add multiple custom headers to the email.

        Args:
            headers_dict: Dictionary of header name-value pairs

        Returns:
            EmailBuilder instance for chaining
        """
        for name, value in headers_dict.items():
            self.header(name, value)
        return self

    def build(self) -> EmailRequest:
        """
        Build and return the final EmailRequest object.

        Returns:
            Validated EmailRequest object

        Raises:
            ValidationError: If the email configuration is invalid
        """
        # Prepare data for EmailRequest
        data = {
            "to": self._to,
            "subject": self._subject,
        }

        # Add optional fields
        if self._from_email:
            data["from_email"] = self._from_email
        if self._cc:
            data["cc"] = self._cc
        if self._bcc:
            data["bcc"] = self._bcc
        if self._reply_to:
            data["reply_to"] = self._reply_to
        if self._html:
            data["html"] = self._html
        if self._text:
            data["text"] = self._text
        if self._template_id:
            data["template_id"] = self._template_id
        if self._attachments:
            data["attachments"] = self._attachments
        if self._tags:
            data["tags"] = self._tags
        if self._personalization:
            data["personalization"] = self._personalization
        if self._precedence_bulk is not None:
            data["precedence_bulk"] = self._precedence_bulk
        if self._send_at:
            data["send_at"] = self._send_at
        if self._in_reply_to:
            data["in_reply_to"] = self._in_reply_to
        if self._references:
            data["references"] = self._references
        if self._settings:
            data["settings"] = self._settings
        if self._headers:
            data["headers"] = self._headers

        # Create and return EmailRequest
        try:
            return EmailRequest(**data)
        except Exception as e:
            raise ValidationError(f"Failed to build email request: {str(e)}")

    def reset(self) -> "EmailBuilder":
        """
        Reset the builder to start fresh.

        Returns:
            EmailBuilder instance for chaining
        """
        self.__init__()
        return self

    def copy(self) -> "EmailBuilder":
        """
        Create a copy of the current builder state.

        Returns:
            New EmailBuilder instance with the same configuration
        """
        new_builder = EmailBuilder()

        # Copy all state
        new_builder._from_email = self._from_email
        new_builder._to = self._to.copy()
        new_builder._cc = self._cc.copy()
        new_builder._bcc = self._bcc.copy()
        new_builder._reply_to = self._reply_to
        new_builder._subject = self._subject
        new_builder._html = self._html
        new_builder._text = self._text
        new_builder._template_id = self._template_id
        new_builder._attachments = self._attachments.copy()
        new_builder._tags = self._tags.copy()
        new_builder._personalization = self._personalization.copy()
        new_builder._precedence_bulk = self._precedence_bulk
        new_builder._send_at = self._send_at
        new_builder._in_reply_to = self._in_reply_to
        new_builder._references = self._references.copy()
        new_builder._settings = self._settings
        new_builder._headers = self._headers.copy()

        return new_builder



"""Email Verification resource"""

from .base import BaseResource
from ..models.base import APIResponse
from ..models.email_verification import (
    EmailVerifyRequest,
    EmailVerifyAsyncRequest,
    EmailVerificationAsyncStatusRequest,
    EmailVerificationListsRequest,
    EmailVerificationGetRequest,
    EmailVerificationCreateRequest,
    EmailVerificationVerifyRequest,
    EmailVerificationResultsRequest,
)
from ..exceptions import ValidationError


class EmailVerification(BaseResource):
    """Resource for managing email verification through the MailerSend API."""

    def verify_email(self, request: EmailVerifyRequest) -> APIResponse:
        """Verify a single email address (synchronous).

        Args:
            request: The email verification request data.

        Returns:
            APIResponse with verification result
        """
        self.logger.debug("Starting verify_email operation")

        # Prepare request body
        body = request.model_dump(exclude_none=True)

        self.logger.debug("Verifying email address: %s", body)

        # Make API call
        response = self.client.request(method="POST", path="email-verification/verify", body=body)

        # Create standardized response
        return self._create_response(response)

    def verify_email_async(self, request: EmailVerifyAsyncRequest) -> APIResponse:
        """Verify a single email address (asynchronous).

        Args:
            request: The async email verification request data.

        Returns:
            APIResponse with verification result
        """
        self.logger.debug("Starting verify_email_async operation")
        self.logger.debug("Async email verify request: %s", request)

        # Prepare request body
        body = request.model_dump(exclude_none=True)

        self.logger.debug("Starting async verification for email: %s", body)

        # Make API call
        response = self.client.request(
            method="POST", path="email-verification/verify-async", body=body
        )

        # Create standardized response
        return self._create_response(response)

    def get_async_status(
        self, request: EmailVerificationAsyncStatusRequest
    ) -> APIResponse:
        """Get the status of an async email verification.

        Args:
            request: The async status request data.

        Returns:
            APIResponse with EmailVerificationAsyncStatusResponse data
        """
        self.logger.debug("Starting get_async_status operation")

        # Prepare API call
        self.logger.debug(
            "Getting async verification status for ID: %s",
            request.email_verification_id,
        )

        # Make API call
        response = self.client.request(
            method="GET",
            path=f"email-verification/verify-async/{request.email_verification_id}",
        )

        # Create standardized response
        return self._create_response(response)

    def list_verifications(self, request: EmailVerificationListsRequest) -> APIResponse:
        """List all email verification lists.

        Args:
            request: The list request data with pagination options.

        Returns:
            APIResponse with list of email verification lists
        """
        self.logger.debug("Starting list_verifications operation")

        # Extract query parameters
        params = request.to_query_params()

        self.logger.debug("Listing email verification lists with params: %s", params)

        # Make API call
        response = self.client.request(
            method="GET", path="email-verification", params=params
        )

        # Create standardized response
        return self._create_response(response)

    def get_verification(self, request: EmailVerificationGetRequest) -> APIResponse:
        """Get a single email verification list.

        Args:
            request: The get verification request data.

        Returns:
            APIResponse with email verification list data
        """
        self.logger.debug("Starting get_verification operation")

        # Prepare API call
        self.logger.debug(
            "Getting email verification list: %s", request.email_verification_id
        )

        # Make API call
        response = self.client.request(
            method="GET", path=f"email-verification/{request.email_verification_id}"
        )

        # Create standardized response
        return self._create_response(response)

    def create_verification(
        self, request: EmailVerificationCreateRequest
    ) -> APIResponse:
        """Create a new email verification list.

        Args:
            request: The create verification request data.

        Returns:
            APIResponse with email verification list data
        """
        self.logger.debug("Starting create_verification operation")

        # Prepare request body
        body = request.model_dump(exclude_none=True)

        self.logger.debug(
            "Creating email verification list: %s with %s emails",
            request.name,
            len(request.emails),
        )

        # Make API call
        response = self.client.request(method="POST", path="email-verification", body=body)

        # Create standardized response
        return self._create_response(response)

    def verify_list(self, request: EmailVerificationVerifyRequest) -> APIResponse:
        """Start verification of an email verification list.

        Args:
            request: The verify list request data.

        Returns:
            APIResponse with verification result
        """
        self.logger.debug("Starting verify_list operation")
        self.logger.debug(
            "Starting verification for list: %s", request.email_verification_id
        )

        # Make API call
        response = self.client.request(
            method="GET",
            path=f"email-verification/{request.email_verification_id}/verify",
        )

        # Create standardized response
        return self._create_response(response)

    def get_results(self, request: EmailVerificationResultsRequest) -> APIResponse:
        """Get verification results for an email verification list.

        Args:
            request: The results request data with optional filters.

        Returns:
            APIResponse with verification results
        """
        self.logger.debug("Starting get_results operation")

        # Extract query parameters
        params = request.to_query_params()

        self.logger.debug(
            "Getting verification results for list: %s with params: %s",
            request.email_verification_id,
            params,
        )

        # Make API call
        response = self.client.request(
            method="GET",
            path=f"email-verification/{request.email_verification_id}/results",
            params=params,
        )

        # Create standardized response
        return self._create_response(response)



"""SMS Webhooks resource."""

from .base import BaseResource
from ..models.sms_webhooks import (
    SmsWebhooksListRequest,
    SmsWebhookGetRequest,
    SmsWebhookCreateRequest,
    SmsWebhookUpdateRequest,
    SmsWebhookDeleteRequest,
)
from ..models.base import APIResponse


class SmsWebhooks(BaseResource):
    """SMS Webhooks resource for MailerSend API."""

    def list_sms_webhooks(self, request: SmsWebhooksListRequest) -> APIResponse:
        """
        List SMS webhooks.

        Args:
            request: SmsWebhooksListRequest object containing query parameters

        Returns:
            APIResponse: Response containing list of SMS webhooks
        """
        params = request.to_query_params()

        self.logger.debug(
            "Listing SMS webhooks for SMS number: %s",
            request.query_params.sms_number_id,
        )

        response = self.client.request(
            method="GET", path="sms-webhooks", params=params
        )

        return self._create_response(response)

    def get_sms_webhook(self, request: SmsWebhookGetRequest) -> APIResponse:
        """
        Get a single SMS webhook.

        Args:
            request: SmsWebhookGetRequest object containing SMS webhook ID

        Returns:
            APIResponse: Response containing SMS webhook details
        """
        self.logger.debug("Getting SMS webhook: %s", request.sms_webhook_id)

        response = self.client.request(
            method="GET", path=f"sms-webhooks/{request.sms_webhook_id}"
        )

        return self._create_response(response)

    def create_sms_webhook(self, request: SmsWebhookCreateRequest) -> APIResponse:
        """
        Create an SMS webhook.

        Args:
            request: SmsWebhookCreateRequest object containing webhook data

        Returns:
            APIResponse: Response containing created SMS webhook
        """
        self.logger.debug(
            "Creating SMS webhook: {request.name} for SMS number: %s",
            request.sms_number_id,
        )

        response = self.client.request(
            method="POST", path="sms-webhooks", body=request.to_request_body()
        )

        return self._create_response(response)

    def update_sms_webhook(self, request: SmsWebhookUpdateRequest) -> APIResponse:
        """
        Update an SMS webhook.

        Args:
            request: SmsWebhookUpdateRequest object containing SMS webhook ID and update data

        Returns:
            APIResponse: Response containing updated SMS webhook
        """
        self.logger.debug("Updating SMS webhook: %s", request.sms_webhook_id)

        response = self.client.request(
            method="PUT",
            path=f"sms-webhooks/{request.sms_webhook_id}",
            body=request.to_request_body(),
        )

        return self._create_response(response)

    def delete_sms_webhook(self, request: SmsWebhookDeleteRequest) -> APIResponse:
        """
        Delete an SMS webhook.

        Args:
            request: SmsWebhookDeleteRequest object containing SMS webhook ID

        Returns:
            APIResponse: Response confirming deletion
        """
        self.logger.debug("Deleting SMS webhook: %s", request.sms_webhook_id)

        response = self.client.request(
            method="DELETE", path=f"sms-webhooks/{request.sms_webhook_id}"
        )

        return self._create_response(response)



"""SMS Recipients models."""

from typing import Optional, List, Dict, Any
from datetime import datetime
from pydantic import BaseModel, Field, field_validator
from enum import Enum

from .base import BaseModel as BaseMailerSendModel


class SmsRecipientStatus(str, Enum):
    """SMS recipient status options."""

    ACTIVE = "active"
    OPT_OUT = "opt_out"


# Query Parameters Models
class SmsRecipientsListQueryParams(BaseModel):
    """Query parameters for listing SMS recipients."""

    status: Optional[SmsRecipientStatus] = Field(
        None, description="Recipient status filter"
    )
    sms_number_id: Optional[str] = Field(None, description="SMS number ID filter")
    page: int = Field(default=1, ge=1, description="Page number")
    limit: int = Field(
        default=25, ge=10, le=100, description="Number of results per page"
    )

    @field_validator("sms_number_id")
    @classmethod
    def validate_sms_number_id(cls, v: Optional[str]) -> Optional[str]:
        """Validate and clean sms_number_id."""
        if v is not None:
            return v.strip()
        return v

    def to_query_params(self) -> Dict[str, Any]:
        """Convert to query parameters dict, excluding None values."""
        params = {}

        if self.status is not None:
            params["status"] = self.status.value
        if self.sms_number_id is not None:
            params["sms_number_id"] = self.sms_number_id
        if self.page != 1:
            params["page"] = self.page
        if self.limit != 25:
            params["limit"] = self.limit

        return params


# Request Models
class SmsRecipientsListRequest(BaseModel):
    """Request model for listing SMS recipients."""

    query_params: SmsRecipientsListQueryParams = Field(
        default_factory=SmsRecipientsListQueryParams
    )

    def to_query_params(self) -> Dict[str, Any]:
        """Convert to query parameters dict."""
        return self.query_params.to_query_params()


class SmsRecipientGetRequest(BaseModel):
    """Request model for getting a single SMS recipient."""

    sms_recipient_id: str = Field(..., min_length=1, description="SMS Recipient ID")

    @field_validator("sms_recipient_id")
    @classmethod
    def validate_sms_recipient_id(cls, v):
        """Validate SMS recipient ID."""
        if not v or not v.strip():
            raise ValueError("SMS recipient ID cannot be empty")
        return v.strip()


class SmsRecipientUpdateRequest(BaseModel):
    """Request model for updating an SMS recipient."""

    sms_recipient_id: str = Field(..., min_length=1, description="SMS Recipient ID")
    status: SmsRecipientStatus = Field(..., description="New recipient status")

    @field_validator("sms_recipient_id")
    @classmethod
    def validate_sms_recipient_id(cls, v):
        """Validate SMS recipient ID."""
        if not v or not v.strip():
            raise ValueError("SMS recipient ID cannot be empty")
        return v.strip()

    def to_request_body(self) -> Dict[str, Any]:
        """Convert to request body dict."""
        return {"status": self.status.value}


# Response Models
class SmsRecipient(BaseMailerSendModel):
    """SMS recipient model."""

    id: str = Field(..., description="SMS recipient ID")
    number: str = Field(..., description="Phone number")
    status: str = Field(..., description="Recipient status")
    created_at: datetime = Field(..., description="Creation timestamp")


class SmsRecipientDetails(SmsRecipient):
    """SMS recipient with detailed information including SMS history."""

    sms: List[Dict[str, Any]] = Field(
        default_factory=list, description="SMS messages sent to this recipient"
    )



"""Identities resource"""

from ..models.identities import (
    IdentityListRequest,
    IdentityCreateRequest,
    IdentityGetRequest,
    IdentityGetByEmailRequest,
    IdentityUpdateRequest,
    IdentityUpdateByEmailRequest,
    IdentityDeleteRequest,
    IdentityDeleteByEmailRequest,
)
from ..models.base import APIResponse
from .base import BaseResource


class IdentitiesResource(BaseResource):
    """Resource for managing sender identities."""

    def list_identities(self, request: IdentityListRequest) -> APIResponse:
        """
        Get a list of sender identities.

        Args:
            request: The identity list request containing filtering and pagination parameters

        Returns:
            APIResponse containing the identities list response
        """
        self.logger.debug("Preparing to list identities with query parameters")

        # Extract query parameters
        params = request.to_query_params()

        self.logger.debug(
            "Making API request to list identities with params: %s", params
        )

        # Make API request
        response = self.client.request(
            method="GET", path="identities", params=params if params else None
        )

        return self._create_response(response)

    def create_identity(self, request: IdentityCreateRequest) -> APIResponse:
        """
        Create a new sender identity.

        Args:
            request: The identity creation request with all required data

        Returns:
            APIResponse containing the created identity response
        """
        self.logger.debug("Preparing to create identity")

        # Build request body
        data = request.model_dump(by_alias=True, exclude_none=True)

        self.logger.debug(
            "Making API request to create identity with data keys: %s",
            list(data.keys()),
        )

        # Make API request
        response = self.client.request(method="POST", path="identities", body=data)

        return self._create_response(response)

    def get_identity(self, request: IdentityGetRequest) -> APIResponse:
        """
        Get a single sender identity by ID.

        Args:
            request: The identity get request with identity ID

        Returns:
            APIResponse containing the identity data
        """
        self.logger.debug("Preparing to get identity with ID: %s", request.identity_id)

        # Make API request
        response = self.client.request(
            method="GET", path=f"identities/{request.identity_id}"
        )

        return self._create_response(response)

    def get_identity_by_email(self, request: IdentityGetByEmailRequest) -> APIResponse:
        """
        Get a single sender identity by email.

        Args:
            request: The identity get by email request

        Returns:
            APIResponse containing the identity data
        """
        self.logger.debug("Preparing to get identity by email: %s", request.email)

        # Make API request
        response = self.client.request(
            method="GET", path=f"identities/email/{request.email}"
        )

        return self._create_response(response)

    def update_identity(self, request: IdentityUpdateRequest) -> APIResponse:
        """
        Update a sender identity by ID.

        Args:
            request: The identity update request with identity ID and update data

        Returns:
            APIResponse containing the updated identity
        """
        self.logger.debug(
            "Preparing to update identity with ID: %s", request.identity_id
        )

        # Build request body, excluding identity_id (goes in URL)
        data = request.model_dump(
            by_alias=True, exclude_none=True, exclude={"identity_id"}
        )

        self.logger.debug(
            "Making API request to update identity with data keys: %s",
            list(data.keys()),
        )

        # Make API request
        response = self.client.request(
            method="PUT",
            path=f"identities/{request.identity_id}",
            body=data if data else None,
        )

        return self._create_response(response)

    def update_identity_by_email(
        self, request: IdentityUpdateByEmailRequest
    ) -> APIResponse:
        """
        Update a sender identity by email.

        Args:
            request: The identity update by email request

        Returns:
            APIResponse containing the updated identity
        """
        self.logger.debug("Preparing to update identity by email: %s", request.email)

        # Build request body, excluding email (goes in URL)
        data = request.model_dump(by_alias=True, exclude_none=True, exclude={"email"})

        self.logger.debug(
            "Making API request to update identity by email with data keys: %s",
            list(data.keys()),
        )

        # Make API request
        response = self.client.request(
            method="PUT",
            path=f"identities/email/{request.email}",
            body=data if data else None,
        )

        return self._create_response(response)

    def delete_identity(self, request: IdentityDeleteRequest) -> APIResponse:
        """
        Delete a sender identity by ID.

        Args:
            request: The identity delete request with identity ID

        Returns:
            APIResponse containing the deletion result
        """
        self.logger.debug(
            "Preparing to delete identity with ID: %s", request.identity_id
        )

        # Make API request
        response = self.client.request(
            method="DELETE", path=f"identities/{request.identity_id}"
        )

        return self._create_response(response)

    def delete_identity_by_email(
        self, request: IdentityDeleteByEmailRequest
    ) -> APIResponse:
        """
        Delete a sender identity by email.

        Args:
            request: The identity delete by email request

        Returns:
            APIResponse containing the deletion result
        """
        self.logger.debug("Preparing to delete identity by email: %s", request.email)

        # Make API request
        response = self.client.request(
            method="DELETE", path=f"identities/email/{request.email}"
        )

        return self._create_response(response)



from typing import List, Dict, Any, Generic, TypeVar, Optional, Union
from pydantic import BaseModel as PydanticBaseModel, ConfigDict
import json

T = TypeVar("T")


class BaseModel(PydanticBaseModel):
    """
    Base model for all MailerSend data models.

    Extends Pydantic's BaseModel with common functionality for all models
    in the MailerSend SDK.
    """

    # Allow extra fields that might be sent by the API but not defined in models
    # Allow field population by alias to support API responses with different naming
    model_config = ConfigDict(validate_by_name=True, extra="ignore")

    @classmethod
    def from_api(cls, data: Dict[str, Any]):
        """
        Create a model instance from API response data.

        Args:
            data: API response data dictionary

        Returns:
            Initialized model instance
        """
        return cls(**data)


class HeaderDict(dict):
    """
    A dictionary that supports both dict['key'] and dict.key access patterns.
    Converts dashes to underscores for attribute access.
    """

    def __getattr__(self, name):
        # Convert underscores back to dashes for header lookup
        header_name = name.replace("_", "-")
        if header_name in self:
            return self[header_name]
        raise AttributeError(f"Header '{header_name}' not found")

    def __setattr__(self, name, value):
        # Convert underscores to dashes and store in dict
        header_name = name.replace("_", "-")
        self[header_name] = value


class APIResponse:
    """
    Unified response container for all API responses.

    This class provides a consistent interface for accessing response data,
    headers, and metadata while maintaining backward compatibility.

    Examples:
        >>> response = client.emails.send(email)
        >>> email_id = response["id"]  # Dict-like access
        >>> email_id = response.id     # Attribute access
        >>> request_id = response.headers["x-request-id"]        # Dict access
        >>> request_id = response.headers.x_request_id           # Attribute access
        >>> request_id = response["headers"]["x-request-id"]     # Nested dict access

        # Handle method name conflicts (items, keys, values, etc.)
        >>> items = response["items"]     # Dict access (recommended)
        >>> items = response.data_items   # data_ prefix access

        # JSON conversion
        >>> json_str = response.to_json()                       # Method call
        >>> json_str = json.dumps(response)                     # Direct serialization
    """

    def __init__(
        self,
        data: Any,
        headers: Dict[str, str],
        status_code: int,
        request_id: Optional[str] = None,
        rate_limit_remaining: Optional[int] = None,
    ):
        self.data = data
        self.headers = HeaderDict(headers)  # Use HeaderDict instead of regular dict
        self.status_code = status_code
        self.request_id = request_id
        self.rate_limit_remaining = rate_limit_remaining

    def __getitem__(self, key):
        """Allow dict-like access to data and object attributes."""
        # First try data dict
        if isinstance(self.data, dict) and key in self.data:
            return self.data[key]
        elif hasattr(self.data, key):
            return getattr(self.data, key)
        # Then try object attributes (like 'headers', 'status_code', etc.)
        elif hasattr(self, key):
            return getattr(self, key)
        else:
            raise KeyError(f"Key '{key}' not found in response data or attributes")

    def __contains__(self, key):
        """Support 'in' operator."""
        if isinstance(self.data, dict) and key in self.data:
            return True
        elif hasattr(self.data, key):
            return True
        elif hasattr(self, key):
            return True
        return False

    def __getattr__(self, name):
        """Allow direct access to data fields for convenience."""
        # Handle data_ prefix for explicit data field access (avoids method conflicts)
        if name.startswith("data_"):
            field_name = name[5:]  # Remove 'data_' prefix
            if isinstance(self.data, dict) and field_name in self.data:
                return self.data[field_name]
            elif hasattr(self.data, field_name):
                return getattr(self.data, field_name)
            raise AttributeError(f"Data field '{field_name}' not found")

        # Avoid conflicts with built-in methods like 'items', 'keys', 'values'
        # These should always refer to the APIResponse methods, not data fields
        if name in ("items", "keys", "values", "get", "to_dict", "to_json"):
            raise AttributeError(
                f"'{self.__class__.__name__}' object has no attribute '{name}'"
            )

        if isinstance(self.data, dict) and name in self.data:
            return self.data[name]
        elif hasattr(self.data, name):
            return getattr(self.data, name)
        raise AttributeError(
            f"'{self.__class__.__name__}' object has no attribute '{name}'"
        )

    def __repr__(self) -> str:
        return f"APIResponse(status_code={self.status_code}, data={self.data})"

    def __str__(self) -> str:
        return str(self.data)

    def __iter__(self):
        """Make the object iterable for json.dumps() support."""
        return iter(self.to_dict().items())

    def default(self, obj):
        """Custom JSON encoder for json.dumps() support."""
        if isinstance(obj, APIResponse):
            return obj.to_dict()
        raise TypeError(f"Object of type {type(obj)} is not JSON serializable")

    @property
    def success(self) -> bool:
        """Whether the request was successful."""
        return 200 <= self.status_code < 300

    @property
    def retry_after(self) -> Optional[int]:
        """Get the recommended retry time in seconds from headers."""
        retry_after = self.headers.get("Retry-After") or self.headers.get("retry-after")
        if retry_after:
            try:
                return int(retry_after)
            except (ValueError, TypeError):
                pass
        return None

    def get(self, key, default=None):
        """Get a value from data with a default fallback (dict-like interface)."""
        try:
            return self[key]
        except KeyError:
            return default

    def to_dict(self) -> Dict[str, Any]:
        """Convert response to a dictionary for debugging or serialization."""
        return {
            "data": self.data,
            "headers": dict(self.headers),
            "status_code": self.status_code,
            "request_id": self.request_id,
            "rate_limit_remaining": self.rate_limit_remaining,
            "success": self.success,
        }

    def to_json(self, indent: Optional[int] = None, **kwargs) -> str:
        """
        Convert response to JSON string.

        Args:
            indent: Number of spaces for indentation (None for compact JSON)
            **kwargs: Additional arguments passed to json.dumps()

        Returns:
            JSON string representation of the response

        Examples:
            >>> response.to_json()                    # Compact JSON
            >>> response.to_json(indent=2)            # Pretty-printed JSON
            >>> response.to_json(ensure_ascii=False)  # Custom json.dumps args
        """
        return json.dumps(self.to_dict(), indent=indent, **kwargs)

    def keys(self):
        """Support dict-like interface for json.dumps()."""
        return self.to_dict().keys()

    def values(self):
        """Support dict-like interface for json.dumps()."""
        return self.to_dict().values()

    def items(self):
        """Support dict-like interface for json.dumps()."""
        return self.to_dict().items()


class ModelList(Generic[T]):
    """
    Container for paginated lists of models returned by the API.

    Provides access to the items along with pagination metadata.
    """

    def __init__(
        self,
        items: List[T],
        meta: Optional[Dict[str, Any]] = None,
        links: Optional[Dict[str, Any]] = None,
    ):
        self.items = items
        self.meta = meta or {}
        self.links = links or {}

    def __iter__(self):
        return iter(self.items)

    def __getitem__(self, index):
        return self.items[index]

    def __len__(self):
        return len(self.items)

    @property
    def total(self) -> int:
        """Total number of items across all pages."""
        return self.meta.get("total", len(self.items))

    @property
    def count(self) -> int:
        """Number of items in the current page."""
        return self.meta.get("count", len(self.items))

    @property
    def per_page(self) -> int:
        """Number of items per page."""
        return self.meta.get("per_page", len(self.items))

    @property
    def current_page(self) -> int:
        """Current page number."""
        return self.meta.get("current_page", 1)

    @property
    def last_page(self) -> int:
        """Last page number."""
        return self.meta.get("last_page", 1)

    @property
    def has_more_pages(self) -> bool:
        """Whether there are more pages available."""
        return self.current_page < self.last_page



"""SMS Recipients resource."""

from .base import BaseResource
from ..models.sms_recipients import (
    SmsRecipientsListRequest,
    SmsRecipientGetRequest,
    SmsRecipientUpdateRequest,
)
from ..models.base import APIResponse


class SmsRecipients(BaseResource):
    """SMS Recipients resource for MailerSend API."""

    def list_sms_recipients(self, request: SmsRecipientsListRequest) -> APIResponse:
        """
        List SMS recipients.

        Args:
            request: SmsRecipientsListRequest object containing query parameters

        Returns:
            APIResponse: Response containing list of SMS recipients
        """
        params = request.to_query_params()

        self.logger.debug(
            "Listing SMS recipients with page: {request.query_params.page}, limit: %s",
            request.query_params.limit,
        )

        response = self.client.request(
            method="GET", path="sms-recipients", params=params
        )

        return self._create_response(response)

    def get_sms_recipient(self, request: SmsRecipientGetRequest) -> APIResponse:
        """
        Get a single SMS recipient.

        Args:
            request: SmsRecipientGetRequest object containing SMS recipient ID

        Returns:
            APIResponse: Response containing SMS recipient details
        """
        self.logger.debug("Getting SMS recipient: %s", request.sms_recipient_id)

        response = self.client.request(
            method="GET", path=f"sms-recipients/{request.sms_recipient_id}"
        )

        return self._create_response(response)

    def update_sms_recipient(self, request: SmsRecipientUpdateRequest) -> APIResponse:
        """
        Update a single SMS recipient.

        Args:
            request: SmsRecipientUpdateRequest object containing SMS recipient ID and new status

        Returns:
            APIResponse: Response containing updated SMS recipient
        """
        self.logger.debug(
            "Updating SMS recipient: {request.sms_recipient_id} to status: %s",
            request.status,
        )

        response = self.client.request(
            method="PUT",
            path=f"sms-recipients/{request.sms_recipient_id}",
            body=request.to_request_body(),
        )

        return self._create_response(response)



"""SMS Messages resource."""

from .base import BaseResource
from ..models.sms_messages import SmsMessagesListRequest, SmsMessageGetRequest
from ..models.base import APIResponse


class SmsMessages(BaseResource):
    """SMS Messages resource for MailerSend API."""

    def list_sms_messages(self, request: SmsMessagesListRequest) -> APIResponse:
        """
        List SMS messages.

        Args:
            request: SmsMessagesListRequest object containing query parameters

        Returns:
            APIResponse: Response containing list of SMS messages
        """
        params = request.to_query_params()

        self.logger.debug(
            "Listing SMS messages with page: %s, limit: %s",
            request.query_params.page,
            request.query_params.limit,
        )

        response = self.client.request(
            method="GET", path="sms-messages", params=params
        )

        return self._create_response(response)

    def get_sms_message(self, request: SmsMessageGetRequest) -> APIResponse:
        """
        Get a single SMS message.

        Args:
            request: SmsMessageGetRequest object containing SMS message ID

        Returns:
            APIResponse: Response containing SMS message details
        """
        self.logger.debug("Getting SMS message: %s", request.sms_message_id)

        response = self.client.request(
            method="GET", path=f"sms-messages/{request.sms_message_id}"
        )

        return self._create_response(response)



"""Domains resource"""

from typing import Optional

from .base import BaseResource
from ..models.domains import (
    DomainListRequest,
    DomainCreateRequest,
    DomainDeleteRequest,
    DomainGetRequest,
    DomainUpdateSettingsRequest,
    DomainRecipientsRequest,
    DomainDnsRecordsRequest,
    DomainVerificationRequest,
    DomainListQueryParams,
)
from ..models.base import APIResponse


class Domains(BaseResource):
    """
    Client for interacting with the MailerSend Domains API.

    Provides methods for managing domains, their settings, recipients,
    DNS records, and verification status.
    """

    def list_domains(self, request: Optional[DomainListRequest] = None) -> APIResponse:
        """
        Retrieve a list of domains.

        Args:
            request: Optional DomainListRequest with filtering and pagination options

        Returns:
            APIResponse with list of domains
        """
        self.logger.debug("Retrieving domains list")

        # Use default query params if no request provided
        if not request:
            query_params = DomainListQueryParams()
            params = query_params.to_query_params()
        else:
            params = request.to_query_params()

        self.logger.debug("Query params: %s", params)

        response = self.client.request(method="GET", path="domains", params=params)

        return self._create_response(response)

    def get_domain(self, request: DomainGetRequest) -> APIResponse:
        """
        Retrieve information about a single domain.

        Args:
            request: DomainGetRequest with domain ID

        Returns:
            APIResponse with domain information
        """
        self.logger.debug("Preparing to get domain")
        self.logger.debug("Requesting domain information for: %s", request.domain_id)

        response = self.client.request(method="GET", path=f"domains/{request.domain_id}")

        return self._create_response(response)

    def create_domain(self, request: DomainCreateRequest) -> APIResponse:
        """
        Create a new domain.

        Args:
            request: DomainCreateRequest with domain creation details

        Returns:
            APIResponse with created domain information
        """
        self.logger.debug("Creating domain: %s", request.name)

        # Convert to request body
        body = request.model_dump(by_alias=True, exclude_none=True)

        self.logger.debug("Request body: %s", body)

        response = self.client.request(method="POST", path="domains", body=body)

        return self._create_response(response)

    def delete_domain(self, request: DomainDeleteRequest) -> APIResponse:
        """
        Delete a domain.

        Args:
            request: DomainDeleteRequest with domain ID to delete

        Returns:
            APIResponse (204 No Content on success)
        """
        self.logger.debug("Preparing to delete domain")
        self.logger.debug("Deleting domain: %s", request.domain_id)

        response = self.client.request(method="DELETE", path=f"domains/{request.domain_id}")

        return self._create_response(response)

    def get_domain_recipients(self, request: DomainRecipientsRequest) -> APIResponse:
        """
        Retrieve recipients for a domain.

        Args:
            request: DomainRecipientsRequest with domain ID and pagination options

        Returns:
            APIResponse with list of domain recipients
        """
        self.logger.debug("Preparing to get domain recipients")
        self.logger.debug("Retrieving recipients for domain: %s", request.domain_id)

        # Convert to query parameters
        params = request.to_query_params()

        self.logger.debug("Query params: %s", params)

        response = self.client.request(
            method="GET", path=f"domains/{request.domain_id}/recipients", params=params
        )

        return self._create_response(response)

    def update_domain_settings(
        self, request: DomainUpdateSettingsRequest
    ) -> APIResponse:
        """
        Update domain settings.

        Args:
            request: DomainUpdateSettingsRequest with domain ID and settings to update

        Returns:
            APIResponse with updated domain information
        """
        self.logger.debug("Preparing to update domain settings")
        self.logger.debug("Updating settings for domain: %s", request.domain_id)

        # Convert to request body, excluding domain_id which goes in URL path
        body = request.model_dump(
            by_alias=True, exclude_none=True, exclude={"domain_id"}
        )

        self.logger.debug("Request body: %s", body)

        response = self.client.request(
            method="PUT", path=f"domains/{request.domain_id}/settings", body=body
        )

        return self._create_response(response)

    def get_domain_dns_records(self, request: DomainDnsRecordsRequest) -> APIResponse:
        """
        Retrieve DNS records for a domain.

        Args:
            request: DomainDnsRecordsRequest with domain ID

        Returns:
            APIResponse with domain DNS records
        """
        self.logger.debug("Preparing to get domain DNS records")
        self.logger.debug("Retrieving DNS records for domain: %s", request.domain_id)

        response = self.client.request(
            method="GET", path=f"domains/{request.domain_id}/dns-records"
        )

        return self._create_response(response)

    def get_domain_verification_status(
        self, request: DomainVerificationRequest
    ) -> APIResponse:
        """
        Retrieve verification status for a domain.

        Args:
            request: DomainVerificationRequest with domain ID

        Returns:
            APIResponse with domain verification status
        """
        self.logger.debug("Preparing to get domain verification status")
        self.logger.debug(
            "Retrieving verification status for domain: %s", request.domain_id
        )

        response = self.client.request(method="GET", path=f"domains/{request.domain_id}/verify")

        return self._create_response(response)



from typing import List, Optional, Literal
from pydantic import field_validator, Field

from .base import BaseModel as MailerSendBaseModel


class SchedulesListQueryParams(MailerSendBaseModel):
    """Model for schedules list query parameters with validation."""

    domain_id: Optional[str] = None
    status: Optional[Literal["scheduled", "sent", "error"]] = None
    page: Optional[int] = Field(default=1, ge=1)
    limit: Optional[int] = Field(default=25, ge=10, le=100)

    @field_validator("domain_id")
    def validate_domain_id(cls, v):
        """Validate domain ID is not empty if provided."""
        if v is not None and not v.strip():
            raise ValueError("Domain ID cannot be empty")
        return v.strip() if v else None

    def to_query_params(self) -> dict:
        """Convert to query parameters for API request."""
        params = {
            "domain_id": self.domain_id,
            "status": self.status,
            "page": self.page,
            "limit": self.limit,
        }

        return {k: v for k, v in params.items() if v is not None}


class SchedulesListRequest(MailerSendBaseModel):
    """Request model for listing scheduled messages."""

    query_params: SchedulesListQueryParams

    def to_query_params(self) -> dict:
        """Convert query parameters for API request."""
        return self.query_params.to_query_params()


class ScheduleGetRequest(MailerSendBaseModel):
    """Request model for getting a single scheduled message."""

    message_id: str

    @field_validator("message_id")
    def validate_message_id(cls, v):
        """Validate message ID is provided and not empty."""
        if not v or not v.strip():
            raise ValueError("Message ID is required")
        return v.strip()


class ScheduleDeleteRequest(MailerSendBaseModel):
    """Request model for deleting a scheduled message."""

    message_id: str

    @field_validator("message_id")
    def validate_message_id(cls, v):
        """Validate message ID is provided and not empty."""
        if not v or not v.strip():
            raise ValueError("Message ID is required")
        return v.strip()



"""Inbound resource"""

from mailersend.models.inbound import (
    InboundListRequest,
    InboundGetRequest,
    InboundCreateRequest,
    InboundUpdateRequest,
    InboundDeleteRequest,
)
from mailersend.models.base import APIResponse
from mailersend.resources.base import BaseResource


class InboundResource(BaseResource):
    """Resource for managing inbound routes."""

    def list(self, request: InboundListRequest) -> APIResponse:
        """
        Get a list of inbound routes.

        Args:
            request: The inbound list request containing filtering and pagination parameters

        Returns:
            APIResponse containing the inbound routes list response
        """
        self.logger.debug("Preparing to list inbound routes with query parameters")

        # Extract query parameters
        params = request.to_query_params()

        self.logger.debug(
            "Making API request to list inbound routes with params: %s", params
        )

        # Make API request
        response = self.client.request(
            method="GET", path="inbound", params=params if params else None
        )

        return self._create_response(response)

    def get(self, request: InboundGetRequest) -> APIResponse:
        """
        Get a single inbound route by ID.

        Args:
            request: The inbound get request with inbound ID

        Returns:
            APIResponse containing the inbound route data
        """
        self.logger.debug(
            "Preparing to get inbound route with ID: %s", request.inbound_id
        )

        # Make API request
        response = self.client.request(
            method="GET", path=f"inbound/{request.inbound_id}"
        )

        return self._create_response(response)

    def create(self, request: InboundCreateRequest) -> APIResponse:
        """
        Create a new inbound route.

        Args:
            request: The inbound create request with all required data

        Returns:
            APIResponse containing the created inbound route response
        """
        self.logger.debug("Preparing to create inbound route")

        # Build request body using model's serialization method
        data = request.to_request_body()

        self.logger.debug(
            "Making API request to create inbound route with data keys: %s",
            list(data.keys()),
        )

        # Make API request
        response = self.client.request(method="POST", path="inbound", body=data)

        return self._create_response(response)

    def update(self, request: InboundUpdateRequest) -> APIResponse:
        """
        Update an existing inbound route.

        Args:
            request: The inbound update request with inbound ID and update data

        Returns:
            APIResponse containing the updated response
        """
        self.logger.debug(
            "Preparing to update inbound route with ID: %s", request.inbound_id
        )

        # Build request body using model's serialization method
        data = request.to_request_body()

        self.logger.debug(
            "Making API request to update inbound route with data keys: %s",
            list(data.keys()),
        )

        # Make API request
        response = self.client.request(
            method="PUT", path=f"inbound/{request.inbound_id}", body=data
        )

        return self._create_response(response)

    def delete(self, request: InboundDeleteRequest) -> APIResponse:
        """
        Delete an inbound route.

        Args:
            request: The inbound delete request with inbound ID

        Returns:
            APIResponse containing the deletion result
        """
        self.logger.debug(
            "Preparing to delete inbound route with ID: %s", request.inbound_id
        )

        # Make API request
        response = self.client.request(
            method="DELETE", path=f"inbound/{request.inbound_id}"
        )

        return self._create_response(response)



"""SMS Webhooks builder for MailerSend SDK."""

from typing import Optional, List

from ..models.sms_webhooks import (
    SmsWebhooksListRequest,
    SmsWebhooksListQueryParams,
    SmsWebhookGetRequest,
    SmsWebhookCreateRequest,
    SmsWebhookUpdateRequest,
    SmsWebhookDeleteRequest,
    SmsWebhookEvent,
)


class SmsWebhooksBuilder:
    """Builder for SMS Webhooks API requests."""

    def __init__(self) -> None:
        """Initialize the SmsWebhooksBuilder."""
        self._sms_number_id: Optional[str] = None
        self._sms_webhook_id: Optional[str] = None
        self._url: Optional[str] = None
        self._name: Optional[str] = None
        self._events: Optional[List[SmsWebhookEvent]] = None
        self._enabled: Optional[bool] = None

    def sms_number_id(self, sms_number_id: str) -> "SmsWebhooksBuilder":
        """
        Set the SMS number ID for listing or creating webhooks.

        Args:
            sms_number_id: SMS number ID

        Returns:
            SmsWebhooksBuilder: Builder instance for method chaining
        """
        self._sms_number_id = sms_number_id
        return self

    def sms_webhook_id(self, sms_webhook_id: str) -> "SmsWebhooksBuilder":
        """
        Set the SMS webhook ID for get, update, and delete operations.

        Args:
            sms_webhook_id: SMS webhook ID

        Returns:
            SmsWebhooksBuilder: Builder instance for method chaining
        """
        self._sms_webhook_id = sms_webhook_id
        return self

    def url(self, url: str) -> "SmsWebhooksBuilder":
        """
        Set the webhook URL for create and update operations.

        Args:
            url: Webhook URL

        Returns:
            SmsWebhooksBuilder: Builder instance for method chaining
        """
        self._url = url
        return self

    def name(self, name: str) -> "SmsWebhooksBuilder":
        """
        Set the webhook name for create and update operations.

        Args:
            name: Webhook name

        Returns:
            SmsWebhooksBuilder: Builder instance for method chaining
        """
        self._name = name
        return self

    def events(self, events: List[SmsWebhookEvent]) -> "SmsWebhooksBuilder":
        """
        Set the events list for create and update operations.

        Args:
            events: List of SMS webhook events

        Returns:
            SmsWebhooksBuilder: Builder instance for method chaining
        """
        self._events = events
        return self

    def add_event(self, event: SmsWebhookEvent) -> "SmsWebhooksBuilder":
        """
        Add a single event to the events list.

        Args:
            event: SMS webhook event to add

        Returns:
            SmsWebhooksBuilder: Builder instance for method chaining
        """
        if self._events is None:
            self._events = []
        if event not in self._events:
            self._events.append(event)
        return self

    def enabled(self, enabled: bool) -> "SmsWebhooksBuilder":
        """
        Set the enabled status for create and update operations.

        Args:
            enabled: Whether webhook is enabled

        Returns:
            SmsWebhooksBuilder: Builder instance for method chaining
        """
        self._enabled = enabled
        return self

    def build_list_request(self) -> SmsWebhooksListRequest:
        """
        Build a request for listing SMS webhooks.

        Returns:
            SmsWebhooksListRequest: Request object for listing SMS webhooks

        Raises:
            ValueError: If SMS number ID is not set
        """
        if self._sms_number_id is None:
            raise ValueError("SMS number ID is required for list request")

        query_params = SmsWebhooksListQueryParams(sms_number_id=self._sms_number_id)
        return SmsWebhooksListRequest(query_params=query_params)

    def build_get_request(self) -> SmsWebhookGetRequest:
        """
        Build a request for getting a single SMS webhook.

        Returns:
            SmsWebhookGetRequest: Request object for getting SMS webhook

        Raises:
            ValueError: If SMS webhook ID is not set
        """
        if self._sms_webhook_id is None:
            raise ValueError("SMS webhook ID is required for get request")

        return SmsWebhookGetRequest(sms_webhook_id=self._sms_webhook_id)

    def build_create_request(self) -> SmsWebhookCreateRequest:
        """
        Build a request for creating an SMS webhook.

        Returns:
            SmsWebhookCreateRequest: Request object for creating SMS webhook

        Raises:
            ValueError: If required fields are not set
        """
        if self._url is None:
            raise ValueError("URL is required for create request")
        if self._name is None:
            raise ValueError("Name is required for create request")
        if self._events is None or len(self._events) == 0:
            raise ValueError("Events list is required for create request")
        if self._sms_number_id is None:
            raise ValueError("SMS number ID is required for create request")

        # Only include enabled if it's explicitly set
        kwargs = {
            "url": self._url,
            "name": self._name,
            "events": self._events,
            "sms_number_id": self._sms_number_id,
        }
        if self._enabled is not None:
            kwargs["enabled"] = self._enabled

        return SmsWebhookCreateRequest(**kwargs)

    def build_update_request(self) -> SmsWebhookUpdateRequest:
        """
        Build a request for updating an SMS webhook.

        Returns:
            SmsWebhookUpdateRequest: Request object for updating SMS webhook

        Raises:
            ValueError: If SMS webhook ID is not set
        """
        if self._sms_webhook_id is None:
            raise ValueError("SMS webhook ID is required for update request")

        return SmsWebhookUpdateRequest(
            sms_webhook_id=self._sms_webhook_id,
            url=self._url,
            name=self._name,
            events=self._events,
            enabled=self._enabled,
        )

    def build_delete_request(self) -> SmsWebhookDeleteRequest:
        """
        Build a request for deleting an SMS webhook.

        Returns:
            SmsWebhookDeleteRequest: Request object for deleting SMS webhook

        Raises:
            ValueError: If SMS webhook ID is not set
        """
        if self._sms_webhook_id is None:
            raise ValueError("SMS webhook ID is required for delete request")

        return SmsWebhookDeleteRequest(sms_webhook_id=self._sms_webhook_id)



"""
Inbound models
"""

from typing import List, Optional
from pydantic import Field, field_validator, model_validator
from mailersend.models.base import BaseModel


# Data Models
class InboundFilter(BaseModel):
    """Represents an inbound filter (catch or match filter)."""

    type: str = Field(..., description="Filter type")
    key: Optional[str] = Field(
        None, description="Filter key (required for match_header)"
    )
    comparer: Optional[str] = Field(None, description="Filter comparer")
    value: Optional[str] = Field(None, description="Filter value")

    @field_validator("type")
    @classmethod
    def validate_type(cls, v):
        """Validate filter type."""
        if v is None:
            return v
        valid_types = [
            "catch_all",
            "catch_recipient",
            "match_all",
            "match_sender",
            "match_domain",
            "match_header",
        ]
        if v not in valid_types:
            raise ValueError(f"Type must be one of: {', '.join(valid_types)}")
        return v

    @field_validator("comparer")
    @classmethod
    def validate_comparer(cls, v):
        """Validate comparer."""
        if v is None:
            return v
        valid_comparers = [
            "equal",
            "not-equal",
            "contains",
            "not-contains",
            "starts-with",
            "ends-with",
            "not-starts-with",
            "not-ends-with",
        ]
        if v not in valid_comparers:
            raise ValueError(f"Comparer must be one of: {', '.join(valid_comparers)}")
        return v

    @field_validator("value")
    @classmethod
    def validate_value(cls, v):
        """Validate value."""
        if v is not None and len(v) > 191:
            raise ValueError("Value cannot exceed 191 characters")
        return v.strip() if v else v

    @field_validator("key")
    @classmethod
    def validate_key(cls, v):
        """Validate key."""
        if v is not None and len(v) > 191:
            raise ValueError("Key cannot exceed 191 characters")
        return v.strip() if v else v


class InboundFilterGroup(BaseModel):
    """Represents a group of inbound filters with type and filters."""

    type: str = Field(..., description="Filter group type")
    filters: Optional[List[dict]] = Field(None, description="List of filter conditions")

    @field_validator("type")
    @classmethod
    def validate_type(cls, v):
        """Validate filter type."""
        valid_types = [
            "catch_all",
            "catch_recipient",
            "match_all",
            "match_sender",
            "match_domain",
            "match_header",
        ]
        if v not in valid_types:
            raise ValueError(f"Type must be one of: {', '.join(valid_types)}")
        return v

    @field_validator("filters")
    @classmethod
    def validate_filters(cls, v):
        """Validate filters."""
        if v is not None and len(v) > 5:
            raise ValueError("Maximum 5 filters allowed")
        return v


class InboundForward(BaseModel):
    """Represents an inbound forward configuration."""

    id: Optional[str] = Field(None, description="Forward ID")
    type: str = Field(..., description="Forward type (email or webhook)")
    value: str = Field(..., description="Forward destination")
    secret: Optional[str] = Field(None, description="Webhook secret")

    @field_validator("type")
    @classmethod
    def validate_type(cls, v):
        """Validate forward type."""
        if v not in ["email", "webhook"]:
            raise ValueError("Type must be either 'email' or 'webhook'")
        return v

    @field_validator("value")
    @classmethod
    def validate_value(cls, v):
        """Validate forward value."""

        if not v or not v.strip():
            raise ValueError("Value is required")
        if len(v) > 191:
            raise ValueError("Value cannot exceed 191 characters")
        return v.strip()


# Request Models
class InboundListQueryParams(BaseModel):
    """Model for inbound list query parameters with validation."""

    page: Optional[int] = Field(default=1, ge=1)
    limit: Optional[int] = Field(default=25, ge=10, le=100)
    domain_id: Optional[str] = None

    def to_query_params(self) -> dict:
        """Convert to query parameters for API request."""
        params = {"page": self.page, "limit": self.limit, "domain_id": self.domain_id}

        return {k: v for k, v in params.items() if v is not None}


class InboundListRequest(BaseModel):
    """Request model for listing inbound routes."""

    query_params: InboundListQueryParams

    def to_query_params(self) -> dict:
        """Convert query parameters for API request."""
        return self.query_params.to_query_params()


class InboundGetRequest(BaseModel):
    """Request model for getting a single inbound route."""

    inbound_id: str = Field(..., description="Inbound route ID")

    @field_validator("inbound_id")
    @classmethod
    def validate_inbound_id(cls, v):
        """Validate inbound ID."""
        if not v or not v.strip():
            raise ValueError("Inbound ID is required")
        return v.strip()


class InboundCreateRequest(BaseModel):
    """Request model for creating an inbound route."""

    domain_id: str = Field(..., description="Domain ID")
    name: str = Field(..., description="Route name")
    domain_enabled: bool = Field(..., description="Whether domain is enabled")
    inbound_domain: Optional[str] = Field(None, description="Inbound domain")
    inbound_priority: Optional[int] = Field(
        None, ge=0, le=100, description="Inbound priority"
    )
    catch_filter: InboundFilterGroup = Field(
        ..., description="Catch filter configuration"
    )
    catch_type: Optional[str] = Field(None, description="Catch type (all or one)")
    match_filter: InboundFilterGroup = Field(
        ..., description="Match filter configuration"
    )
    match_type: Optional[str] = Field(None, description="Match type (all or one)")
    forwards: List[InboundForward] = Field(..., description="Forward configurations")

    @field_validator("domain_id")
    @classmethod
    def validate_domain_id(cls, v):
        """Validate domain ID."""
        if not v or not v.strip():
            raise ValueError("Domain ID is required")
        return v.strip()

    @field_validator("name")
    @classmethod
    def validate_name(cls, v):
        """Validate name."""
        if not v or not v.strip():
            raise ValueError("Name is required")
        if len(v) > 191:
            raise ValueError("Name cannot exceed 191 characters")
        return v.strip()

    @field_validator("inbound_domain")
    @classmethod
    def validate_inbound_domain(cls, v):
        """Validate inbound domain."""
        if v is not None:
            if len(v) > 191:
                raise ValueError("Inbound domain cannot exceed 191 characters")
            return v.strip()
        return v

    @field_validator("catch_type")
    @classmethod
    def validate_catch_type(cls, v):
        """Validate catch type."""
        if v is not None and v not in ["all", "one"]:
            raise ValueError("Catch type must be 'all' or 'one'")
        return v

    @field_validator("match_type")
    @classmethod
    def validate_match_type(cls, v):
        """Validate match type."""
        if v is not None and v not in ["all", "one"]:
            raise ValueError("Match type must be 'all' or 'one'")
        return v

    @field_validator("forwards")
    @classmethod
    def validate_forwards(cls, v):
        """Validate forwards."""
        if not v:
            raise ValueError("At least one forward is required")
        if len(v) > 5:
            raise ValueError("Maximum 5 forwards allowed")

        # Check for distinct values
        values = [forward.value for forward in v]
        if len(values) != len(set(values)):
            raise ValueError("Forward values must be distinct")

        return v

    @model_validator(mode="after")
    def validate_conditional_fields(self):
        """Validate conditional fields."""
        if self.domain_enabled:
            if not self.inbound_domain:
                raise ValueError("Inbound domain is required when domain is enabled")
            if self.inbound_priority is None:
                raise ValueError("Inbound priority is required when domain is enabled")

        return self

    def to_request_body(self) -> dict:
        """Convert to API request body with proper serialization."""
        data = self.model_dump(by_alias=True, exclude_none=True)

        # Exclude 'id' from forwards for creation
        if "forwards" in data and data["forwards"]:
            data["forwards"] = [
                {k: v for k, v in forward.items() if k != "id"}
                for forward in data["forwards"]
            ]

        return data


class InboundUpdateRequest(BaseModel):
    """Request model for updating an inbound route."""

    inbound_id: str = Field(..., description="Inbound route ID")
    name: str = Field(..., description="Route name")
    domain_enabled: bool = Field(..., description="Whether domain is enabled")
    inbound_domain: Optional[str] = Field(None, description="Inbound domain")
    inbound_priority: Optional[int] = Field(
        None, ge=0, le=100, description="Inbound priority"
    )
    catch_filter: InboundFilterGroup = Field(
        ..., description="Catch filter configuration"
    )
    catch_type: Optional[str] = Field(None, description="Catch type (all or one)")
    match_filter: InboundFilterGroup = Field(
        ..., description="Match filter configuration"
    )
    match_type: Optional[str] = Field(None, description="Match type (all or one)")
    forwards: List[InboundForward] = Field(..., description="Forward configurations")

    @field_validator("inbound_id")
    @classmethod
    def validate_inbound_id(cls, v):
        """Validate inbound ID."""
        if not v or not v.strip():
            raise ValueError("Inbound ID is required")
        return v.strip()

    @field_validator("name")
    @classmethod
    def validate_name(cls, v):
        """Validate name."""
        if not v or not v.strip():
            raise ValueError("Name is required")
        if len(v) > 191:
            raise ValueError("Name cannot exceed 191 characters")
        return v.strip()

    @field_validator("inbound_domain")
    @classmethod
    def validate_inbound_domain(cls, v):
        """Validate inbound domain."""
        if v is not None:
            if len(v) > 191:
                raise ValueError("Inbound domain cannot exceed 191 characters")
            return v.strip()
        return v

    @field_validator("catch_type")
    @classmethod
    def validate_catch_type(cls, v):
        """Validate catch type."""
        if v is not None and v not in ["all", "one"]:
            raise ValueError("Catch type must be 'all' or 'one'")
        return v

    @field_validator("match_type")
    @classmethod
    def validate_match_type(cls, v):
        """Validate match type."""
        if v is not None and v not in ["all", "one"]:
            raise ValueError("Match type must be 'all' or 'one'")
        return v

    @field_validator("forwards")
    @classmethod
    def validate_forwards(cls, v):
        """Validate forwards."""
        if not v:
            raise ValueError("At least one forward is required")
        if len(v) > 5:
            raise ValueError("Maximum 5 forwards allowed")

        # Check for distinct values
        values = [forward.value for forward in v]
        if len(values) != len(set(values)):
            raise ValueError("Forward values must be distinct")

        return v

    @model_validator(mode="after")
    def validate_conditional_fields(self):
        """Validate conditional fields."""
        if self.domain_enabled:
            if not self.inbound_domain:
                raise ValueError("Inbound domain is required when domain is enabled")
            if self.inbound_priority is None:
                raise ValueError("Inbound priority is required when domain is enabled")

        return self

    def to_request_body(self) -> dict:
        """Convert to API request body with proper serialization."""
        data = self.model_dump(by_alias=True, exclude_none=True, exclude={"inbound_id"})

        # Exclude 'id' from forwards for update
        if "forwards" in data and data["forwards"]:
            data["forwards"] = [
                {k: v for k, v in forward.items() if k != "id"}
                for forward in data["forwards"]
            ]

        return data


class InboundDeleteRequest(BaseModel):
    """Request model for deleting an inbound route."""

    inbound_id: str = Field(..., description="Inbound route ID")

    @field_validator("inbound_id")
    @classmethod
    def validate_inbound_id(cls, v):
        """Validate inbound ID."""
        if not v or not v.strip():
            raise ValueError("Inbound ID is required")
        return v.strip()



from typing import List, Optional, Literal
from pydantic import Field, field_validator, model_validator

from .base import BaseModel as MailerSendBaseModel


class AnalyticsRequest(MailerSendBaseModel):
    """
    Request model for Analytics API endpoints.

    This model handles all analytics endpoints since they share similar parameters:
    - Activity data by date
    - Opens by country
    - Opens by user-agent name
    - Opens by reading environment
    """

    domain_id: Optional[str] = None
    recipient_id: Optional[List[str]] = Field(None, alias="recipient_id[]")
    date_from: int
    date_to: int
    tags: Optional[List[str]] = Field(None, alias="tags[]")

    # Date endpoint specific parameters
    group_by: Optional[Literal["days", "weeks", "months", "years"]] = "days"
    event: Optional[
        List[
            Literal[
                "queued",
                "sent",
                "delivered",
                "soft_bounced",
                "hard_bounced",
                "opened",
                "clicked",
                "unsubscribed",
                "spam_complaints",
                "survey_opened",
                "survey_submitted",
                "opened_unique",
                "clicked_unique",
            ]
        ]
    ] = Field(None, alias="event[]")

    @field_validator("date_from", "date_to")
    def validate_timestamps(cls, v):
        """Validate that timestamps are positive integers."""
        if v <= 0:
            raise ValueError("Timestamp must be a positive integer")
        return v

    @model_validator(mode="after")
    def validate_date_range(cls, v):
        """Validate that date_from is before date_to."""
        if v.date_from >= v.date_to:
            raise ValueError("date_from must be lower than date_to")
        return v

    @field_validator("recipient_id")
    def validate_recipient_count(cls, v):
        """Validate recipient count limit."""
        if v and len(v) > 50:
            raise ValueError("Maximum 50 recipients are allowed")
        return v

    @field_validator("tags")
    def validate_tags(cls, v):
        """Validate tags format."""
        if v:
            for tag in v:
                if not isinstance(tag, str) or not tag.strip():
                    raise ValueError("All tags must be non-empty strings")
        return v



"""Analytics resource"""

from typing import Dict, Any, Optional

from .base import BaseResource
from ..models.analytics import AnalyticsRequest
from ..models.base import APIResponse


class Analytics(BaseResource):
    """
    Client for interacting with the MailerSend Analytics API.

    Provides methods for retrieving analytics data grouped by date, country,
    user agent, and reading environment.
    """

    def get_activity_by_date(self, request: AnalyticsRequest) -> APIResponse:
        """
        Retrieve analytics data grouped by date.

        Args:
            request: AnalyticsRequest with date range and filtering options

        Returns:
            APIResponse with activity data grouped by date
        """
        self.logger.debug("Retrieving analytics data by date")

        # Convert to query parameters
        params = self._build_query_params(request)

        self.logger.info("Requesting analytics data by date")
        self.logger.debug("Query params: %s", params)

        response = self.client.request("GET", "analytics/date", params=params)

        return self._create_response(response)

    def get_opens_by_country(self, request: AnalyticsRequest) -> APIResponse:
        """
        Retrieve analytics data grouped by country.

        Args:
            request: AnalyticsRequest with date range and filtering options

        Returns:
            APIResponse with opens data grouped by country
        """
        self.logger.debug("Retrieving analytics data by country")

        # Convert to query parameters (exclude event and group_by for country endpoint)
        params = self._build_query_params(request, exclude_fields=["event", "group_by"])

        self.logger.info("Requesting analytics data by country")
        self.logger.debug("Query params: %s", params)

        response = self.client.request("GET", "analytics/country", params=params)

        return self._create_response(response)

    def get_opens_by_user_agent(self, request: AnalyticsRequest) -> APIResponse:
        """
        Retrieve analytics data grouped by user agent name.

        Args:
            request: AnalyticsRequest with date range and filtering options

        Returns:
            APIResponse with opens data grouped by user agent
        """
        self.logger.debug("Retrieving analytics data by user agent")

        # Convert to query parameters (exclude event and group_by for user agent endpoint)
        params = self._build_query_params(request, exclude_fields=["event", "group_by"])

        self.logger.info("Requesting analytics data by user agent")
        self.logger.debug("Query params: %s", params)

        response = self.client.request("GET", "analytics/ua-name", params=params)

        return self._create_response(response)

    def get_opens_by_reading_environment(
        self, request: AnalyticsRequest
    ) -> APIResponse:
        """
        Retrieve analytics data grouped by reading environment.

        Args:
            request: AnalyticsRequest with date range and filtering options

        Returns:
            APIResponse with opens data grouped by reading environment
        """
        self.logger.debug("Retrieving analytics data by reading environment")

        # Convert to query parameters (exclude event and group_by for reading environment endpoint)
        params = self._build_query_params(request, exclude_fields=["event", "group_by"])

        self.logger.info("Requesting analytics data by reading environment")
        self.logger.debug("Query params: %s", params)

        response = self.client.request("GET", "analytics/ua-type", params=params)

        return self._create_response(response)

    def _build_query_params(
        self, request: AnalyticsRequest, exclude_fields: Optional[list] = None
    ) -> Dict[str, Any]:
        """
        Build query parameters from AnalyticsRequest.

        Args:
            request: The AnalyticsRequest object
            exclude_fields: List of fields to exclude from the params

        Returns:
            Dictionary of query parameters
        """
        exclude_fields = exclude_fields or []

        # Convert model to dict with aliases
        params = request.model_dump(by_alias=True, exclude_none=True)

        # Remove excluded fields (including their array versions)
        for field in exclude_fields:
            params.pop(field, None)
            params.pop(f"{field}[]", None)

        return params



"""SMS Messages builder."""

from mailersend.models.sms_messages import (
    SmsMessagesListRequest,
    SmsMessagesListQueryParams,
    SmsMessageGetRequest,
)
from mailersend.exceptions import ValidationError


class SmsMessagesBuilder:
    """Builder for SMS Messages requests."""

    def __init__(self):
        """Initialize the builder."""
        self._sms_message_id = None
        self._page = 1
        self._limit = 25

    def sms_message_id(self, sms_message_id: str) -> "SmsMessagesBuilder":
        """
        Set the SMS message ID.

        Args:
            sms_message_id: SMS message ID

        Returns:
            SmsMessagesBuilder: Builder instance for method chaining
        """
        self._sms_message_id = sms_message_id
        return self

    def page(self, page: int) -> "SmsMessagesBuilder":
        """
        Set the page number.

        Args:
            page: Page number (must be >= 1)

        Returns:
            SmsMessagesBuilder: Builder instance for method chaining

        Raises:
            ValidationError: If page is less than 1
        """
        if page < 1:
            raise ValidationError("Page must be >= 1")
        self._page = page
        return self

    def limit(self, limit: int) -> "SmsMessagesBuilder":
        """
        Set the limit.

        Args:
            limit: Number of results per page (must be between 10 and 100)

        Returns:
            SmsMessagesBuilder: Builder instance for method chaining

        Raises:
            ValidationError: If limit is not between 10 and 100
        """
        if limit < 10 or limit > 100:
            raise ValidationError("Limit must be between 10 and 100")
        self._limit = limit
        return self

    def build_sms_messages_list(self) -> SmsMessagesListRequest:
        """
        Build an SMS messages list request.

        Returns:
            SmsMessagesListRequest: Request object for listing SMS messages
        """
        query_params = SmsMessagesListQueryParams(page=self._page, limit=self._limit)

        return SmsMessagesListRequest(query_params=query_params)

    def build_sms_message_get(self) -> SmsMessageGetRequest:
        """
        Build an SMS message get request.

        Returns:
            SmsMessageGetRequest: Request object for getting SMS message

        Raises:
            ValidationError: If SMS message ID is not set
        """
        if not self._sms_message_id:
            raise ValidationError("SMS message ID is required")

        return SmsMessageGetRequest(sms_message_id=self._sms_message_id)



"""
Analytics Builder - Fluent API for constructing analytics requests.

This module provides the AnalyticsBuilder class which offers a chainable,
developer-friendly API for building analytics requests with intelligent
defaults, date handling, and validation.
"""

from datetime import datetime, timezone, timedelta
from typing import List, Optional, Literal

from ..models.analytics import AnalyticsRequest
from ..exceptions import ValidationError


class AnalyticsBuilder:
    """
    Fluent builder for constructing AnalyticsRequest objects.

    Provides a chainable API that makes it easy to build analytics requests
    for all analytics endpoints with intelligent date handling and validation.

    Examples:
        >>> # Activity data by date
        >>> request = (AnalyticsBuilder()
        ...     .date_range_days(7)  # Last 7 days
        ...     .events("sent", "delivered", "opened")
        ...     .group_by("days")
        ...     .tags("newsletter", "marketing")
        ...     .build())

        >>> # Opens by country for specific domain
        >>> request = (AnalyticsBuilder()
        ...     .domain_id("domain-id-123")
        ...     .date_from_timestamp(1443651141)
        ...     .date_to_timestamp(1443661141)
        ...     .tags("campaign-2024")
        ...     .build())

        >>> # Custom date range with datetime objects
        >>> request = (AnalyticsBuilder()
        ...     .date_from_datetime(datetime(2024, 1, 1))
        ...     .date_to_datetime(datetime(2024, 1, 31))
        ...     .events("opened", "clicked")
        ...     .group_by("weeks")
        ...     .build())
    """

    def __init__(self):
        """Initialize a new AnalyticsBuilder."""
        self._domain_id: Optional[str] = None
        self._recipient_id: List[str] = []
        self._date_from: Optional[int] = None
        self._date_to: Optional[int] = None
        self._tags: List[str] = []
        self._group_by: str = "days"
        self._event: List[str] = []

    def domain_id(self, domain_id: str) -> "AnalyticsBuilder":
        """
        Set the domain ID to filter analytics data.

        Args:
            domain_id: The domain ID to filter by

        Returns:
            AnalyticsBuilder instance for chaining
        """
        self._domain_id = domain_id
        return self

    def recipient(self, recipient_id: str) -> "AnalyticsBuilder":
        """
        Add a recipient ID to filter analytics data.

        Args:
            recipient_id: The recipient ID to add

        Returns:
            AnalyticsBuilder instance for chaining
        """
        if len(self._recipient_id) >= 50:
            raise ValidationError("Maximum 50 recipients are allowed")
        self._recipient_id.append(recipient_id)
        return self

    def recipients(self, *recipient_ids: str) -> "AnalyticsBuilder":
        """
        Add multiple recipient IDs to filter analytics data.

        Args:
            recipient_ids: The recipient IDs to add

        Returns:
            AnalyticsBuilder instance for chaining
        """
        for recipient_id in recipient_ids:
            self.recipient(recipient_id)
        return self

    def date_from(self, timestamp: int) -> "AnalyticsBuilder":
        """
        Set the start date using a Unix timestamp.

        Args:
            timestamp: Unix timestamp (UTC)

        Returns:
            AnalyticsBuilder instance for chaining
        """
        if timestamp <= 0:
            raise ValidationError("Timestamp must be a positive integer")
        self._date_from = timestamp
        return self

    def date_to(self, timestamp: int) -> "AnalyticsBuilder":
        """
        Set the end date using a Unix timestamp.

        Args:
            timestamp: Unix timestamp (UTC)

        Returns:
            AnalyticsBuilder instance for chaining
        """
        if timestamp <= 0:
            raise ValidationError("Timestamp must be a positive integer")
        self._date_to = timestamp
        return self

    def date_from_datetime(self, dt: datetime) -> "AnalyticsBuilder":
        """
        Set the start date using a datetime object.

        Args:
            dt: Datetime object (will be converted to UTC)

        Returns:
            AnalyticsBuilder instance for chaining
        """
        if dt.tzinfo is None:
            dt = dt.replace(tzinfo=timezone.utc)
        self._date_from = int(dt.timestamp())
        return self

    def date_to_datetime(self, dt: datetime) -> "AnalyticsBuilder":
        """
        Set the end date using a datetime object.

        Args:
            dt: Datetime object (will be converted to UTC)

        Returns:
            AnalyticsBuilder instance for chaining
        """
        if dt.tzinfo is None:
            dt = dt.replace(tzinfo=timezone.utc)
        self._date_to = int(dt.timestamp())
        return self

    def date_range(self, date_from: datetime, date_to: datetime) -> "AnalyticsBuilder":
        """
        Set both start and end dates using datetime objects.

        Args:
            date_from: Start datetime
            date_to: End datetime

        Returns:
            AnalyticsBuilder instance for chaining
        """
        self.date_from_datetime(date_from)
        self.date_to_datetime(date_to)
        return self

    def date_range_days(
        self, days: int, end_date: Optional[datetime] = None
    ) -> "AnalyticsBuilder":
        """
        Set date range for the last N days.

        Args:
            days: Number of days to look back
            end_date: End date (defaults to now)

        Returns:
            AnalyticsBuilder instance for chaining
        """
        if end_date is None:
            end_date = datetime.now(timezone.utc)
        elif end_date.tzinfo is None:
            end_date = end_date.replace(tzinfo=timezone.utc)

        start_date = end_date - timedelta(days=days)
        return self.date_range(start_date, end_date)

    def date_range_weeks(
        self, weeks: int, end_date: Optional[datetime] = None
    ) -> "AnalyticsBuilder":
        """
        Set date range for the last N weeks.

        Args:
            weeks: Number of weeks to look back
            end_date: End date (defaults to now)

        Returns:
            AnalyticsBuilder instance for chaining
        """
        return self.date_range_days(weeks * 7, end_date)

    def date_range_months(
        self, months: int, end_date: Optional[datetime] = None
    ) -> "AnalyticsBuilder":
        """
        Set date range for the last N months (approximate).

        Args:
            months: Number of months to look back
            end_date: End date (defaults to now)

        Returns:
            AnalyticsBuilder instance for chaining
        """
        return self.date_range_days(months * 30, end_date)

    def today(self) -> "AnalyticsBuilder":
        """
        Set date range for today (00:00 to now).

        Returns:
            AnalyticsBuilder instance for chaining
        """
        now = datetime.now(timezone.utc)
        start_of_day = now.replace(hour=0, minute=0, second=0, microsecond=0)
        return self.date_range(start_of_day, now)

    def yesterday(self) -> "AnalyticsBuilder":
        """
        Set date range for yesterday (full day).

        Returns:
            AnalyticsBuilder instance for chaining
        """
        now = datetime.now(timezone.utc)
        end_of_yesterday = now.replace(hour=0, minute=0, second=0, microsecond=0)
        start_of_yesterday = end_of_yesterday - timedelta(days=1)
        return self.date_range(start_of_yesterday, end_of_yesterday)

    def this_week(self) -> "AnalyticsBuilder":
        """
        Set date range for this week (Monday to now).

        Returns:
            AnalyticsBuilder instance for chaining
        """
        now = datetime.now(timezone.utc)
        days_since_monday = now.weekday()
        start_of_week = now - timedelta(days=days_since_monday)
        start_of_week = start_of_week.replace(hour=0, minute=0, second=0, microsecond=0)
        return self.date_range(start_of_week, now)

    def this_month(self) -> "AnalyticsBuilder":
        """
        Set date range for this month (1st to now).

        Returns:
            AnalyticsBuilder instance for chaining
        """
        now = datetime.now(timezone.utc)
        start_of_month = now.replace(day=1, hour=0, minute=0, second=0, microsecond=0)
        return self.date_range(start_of_month, now)

    def tag(self, *tags: str) -> "AnalyticsBuilder":
        """
        Add tags to filter analytics data.

        Args:
            tags: Tag names to add

        Returns:
            AnalyticsBuilder instance for chaining
        """
        for tag in tags:
            if not tag.strip():
                raise ValidationError("Tags cannot be empty")
            if tag not in self._tags:
                self._tags.append(tag)
        return self

    def tags(self, *tags: str) -> "AnalyticsBuilder":
        """
        Alias for tag() method for better readability.

        Args:
            tags: Tag names to add

        Returns:
            AnalyticsBuilder instance for chaining
        """
        return self.tag(*tags)

    def group_by(
        self, group_by: Literal["days", "weeks", "months", "years"]
    ) -> "AnalyticsBuilder":
        """
        Set how to group date-based analytics data.

        Args:
            group_by: Grouping option (days, weeks, months, years)

        Returns:
            AnalyticsBuilder instance for chaining
        """
        if group_by not in ["days", "weeks", "months", "years"]:
            raise ValidationError("group_by must be one of: days, weeks, months, years")
        self._group_by = group_by
        return self

    def event(self, *events: str) -> "AnalyticsBuilder":
        """
        Add events to track in date-based analytics.

        Args:
            events: Event names to track

        Returns:
            AnalyticsBuilder instance for chaining
        """
        valid_events = [
            "queued",
            "sent",
            "delivered",
            "soft_bounced",
            "hard_bounced",
            "opened",
            "clicked",
            "unsubscribed",
            "spam_complaints",
            "survey_opened",
            "survey_submitted",
            "opened_unique",
            "clicked_unique",
        ]

        for event in events:
            if event not in valid_events:
                raise ValidationError(
                    f"Invalid event: {event}. Valid events: {', '.join(valid_events)}"
                )
            if event not in self._event:
                self._event.append(event)
        return self

    def events(self, *events: str) -> "AnalyticsBuilder":
        """
        Alias for event() method for better readability.

        Args:
            events: Event names to track

        Returns:
            AnalyticsBuilder instance for chaining
        """
        return self.event(*events)

    def all_events(self) -> "AnalyticsBuilder":
        """
        Track all available events.

        Returns:
            AnalyticsBuilder instance for chaining
        """
        return self.events(
            "queued",
            "sent",
            "delivered",
            "soft_bounced",
            "hard_bounced",
            "opened",
            "clicked",
            "unsubscribed",
            "spam_complaints",
            "survey_opened",
            "survey_submitted",
            "opened_unique",
            "clicked_unique",
        )

    def delivery_events(self) -> "AnalyticsBuilder":
        """
        Track delivery-related events.

        Returns:
            AnalyticsBuilder instance for chaining
        """
        return self.events(
            "queued", "sent", "delivered", "soft_bounced", "hard_bounced"
        )

    def engagement_events(self) -> "AnalyticsBuilder":
        """
        Track engagement-related events.

        Returns:
            AnalyticsBuilder instance for chaining
        """
        return self.events("opened", "clicked", "opened_unique", "clicked_unique")

    def negative_events(self) -> "AnalyticsBuilder":
        """
        Track negative feedback events.

        Returns:
            AnalyticsBuilder instance for chaining
        """
        return self.events("unsubscribed", "spam_complaints")

    def build(self) -> AnalyticsRequest:
        """
        Build the final AnalyticsRequest object.

        Returns:
            Validated AnalyticsRequest instance

        Raises:
            ValidationError: If required fields are missing or invalid
        """
        if self._date_from is None:
            raise ValidationError("date_from is required")
        if self._date_to is None:
            raise ValidationError("date_to is required")

        return AnalyticsRequest(
            domain_id=self._domain_id,
            recipient_id=self._recipient_id if self._recipient_id else None,
            date_from=self._date_from,
            date_to=self._date_to,
            tags=self._tags if self._tags else None,
            group_by=self._group_by,
            event=self._event if self._event else None,
        )

    def reset(self) -> "AnalyticsBuilder":
        """
        Reset the builder to initial state.

        Returns:
            AnalyticsBuilder instance for chaining
        """
        self._domain_id = None
        self._recipient_id = []
        self._date_from = None
        self._date_to = None
        self._tags = []
        self._group_by = "days"
        self._event = []
        return self

    def copy(self) -> "AnalyticsBuilder":
        """
        Create a copy of the current builder state.

        Returns:
            New AnalyticsBuilder instance with copied state
        """
        new_builder = AnalyticsBuilder()
        new_builder._domain_id = self._domain_id
        new_builder._recipient_id = self._recipient_id.copy()
        new_builder._date_from = self._date_from
        new_builder._date_to = self._date_to
        new_builder._tags = self._tags.copy()
        new_builder._group_by = self._group_by
        new_builder._event = self._event.copy()
        return new_builder



"""Schedules resource"""

from .base import BaseResource
from ..models.schedules import (
    SchedulesListRequest,
    ScheduleGetRequest,
    ScheduleDeleteRequest,
)
from ..models.base import APIResponse


class Schedules(BaseResource):
    """
    Client for interacting with the MailerSend Message Schedules API.

    Provides methods for managing scheduled messages.
    """

    def list_schedules(self, request: SchedulesListRequest) -> APIResponse:
        """
        Retrieve a list of scheduled messages.

        Args:
            request: SchedulesListRequest with filtering and pagination options

        Returns:
            APIResponse containing the schedules list response
        """
        self.logger.debug("Preparing to list scheduled messages with query parameters")

        # Extract query parameters
        params = request.to_query_params()

        self.logger.debug(
            "Making API request to list scheduled messages with params: %s", params
        )

        # Make API request
        response = self.client.request(
            method="GET",
            path="message-schedules",
            params=params if params else None,
        )

        return self._create_response(response)

    def get_schedule(self, request: ScheduleGetRequest) -> APIResponse:
        """
        Retrieve information about a single scheduled message.

        Args:
            request: ScheduleGetRequest with message ID

        Returns:
            APIResponse containing the schedule response
        """
        self.logger.debug(
            "Preparing to get scheduled message with ID: %s", request.message_id
        )

        # Make API request
        response = self.client.request(
            method="GET", path=f"message-schedules/{request.message_id}"
        )

        return self._create_response(response)

    def delete_schedule(self, request: ScheduleDeleteRequest) -> APIResponse:
        """
        Delete a scheduled message.

        Args:
            request: ScheduleDeleteRequest with message ID to delete

        Returns:
            APIResponse (204 No Content on success)
        """
        self.logger.debug(
            "Preparing to delete scheduled message with ID: %s", request.message_id
        )

        # Make API request
        response = self.client.request(
            method="DELETE", path=f"message-schedules/{request.message_id}"
        )

        return self._create_response(response)



"""
SMS Activity API resource.
"""

from .base import BaseResource
from ..models.sms_activity import SmsActivityListRequest, SmsMessageGetRequest
from ..models.base import APIResponse


class SmsActivity(BaseResource):
    """Resource for SMS Activity API endpoints."""

    def list(self, request: SmsActivityListRequest) -> APIResponse:
        """
        Get a list of SMS activities.

        Args:
            request: SMS activity list request

        Returns:
            API response with SMS activities

        Raises:
            ValidationError: If request is invalid
        """
        self.logger.debug("Preparing to list SMS activities")

        # Convert to query parameters
        params = request.to_query_params()

        self.logger.debug("Listing SMS activities with params: %s", params)

        # Make API request
        response = self.client.request(
            method="GET", path="sms-activity", params=params
        )

        return self._create_response(response)

    def get(self, request: SmsMessageGetRequest) -> APIResponse:
        """
        Get activity of a single SMS message.

        Args:
            request: SMS message get request

        Returns:
            API response with SMS message activity
        """
        self.logger.debug("Getting SMS message activity: %s", request.sms_message_id)

        # Make API request
        response = self.client.request(
            method="GET", path=f"sms-messages/{request.sms_message_id}"
        )

        return self._create_response(response)



"""Templates API models for MailerSend SDK."""

from typing import List, Optional, Dict, Any
from pydantic import Field, field_validator

from .base import BaseModel as MailerSendBaseModel


# Query Parameters Models
class TemplatesListQueryParams(MailerSendBaseModel):
    """Query parameters for listing templates."""

    domain_id: Optional[str] = None
    page: int = Field(default=1, ge=1)
    limit: int = Field(default=25, ge=10, le=100)

    @field_validator("domain_id")
    @classmethod
    def validate_domain_id(cls, v: Optional[str]) -> Optional[str]:
        """Validate and clean domain_id."""
        if v is not None:
            return v.strip()
        return v

    def to_query_params(self) -> Dict[str, Any]:
        """Convert to query parameters dictionary, excluding None values."""
        params = {}

        if self.domain_id is not None:
            params["domain_id"] = self.domain_id

        # Always include page and limit (they have defaults)
        params["page"] = self.page
        params["limit"] = self.limit

        return params


# Request Models
class TemplatesListRequest(MailerSendBaseModel):
    """Request model for listing templates."""

    query_params: TemplatesListQueryParams = Field(
        default_factory=TemplatesListQueryParams
    )

    def to_query_params(self) -> Dict[str, Any]:
        """Convert to query parameters dictionary."""
        return self.query_params.to_query_params()


class TemplateGetRequest(MailerSendBaseModel):
    """Request model for getting a single template."""

    template_id: str

    @field_validator("template_id")
    @classmethod
    def validate_template_id(cls, v: str) -> str:
        """Validate template ID is provided and not empty."""
        if not v or not v.strip():
            raise ValueError("Template ID is required")
        return v.strip()


class TemplateDeleteRequest(MailerSendBaseModel):
    """Request model for deleting a template."""

    template_id: str

    @field_validator("template_id")
    @classmethod
    def validate_template_id(cls, v: str) -> str:
        """Validate template ID is provided and not empty."""
        if not v or not v.strip():
            raise ValueError("Template ID is required")
        return v.strip()



"""Activity resource"""

from .base import BaseResource
from ..models.activity import ActivityRequest, SingleActivityRequest
from ..models.base import APIResponse


class Activity(BaseResource):
    """
    Client for interacting with the MailerSend Activity API.
    """

    def get(self, request: ActivityRequest) -> APIResponse:
        """
        Get activity data for a domain.

        Args:
            request: A fully-validated ActivityRequest object

        Returns:
            APIResponse with activity data and metadata
        """
        self.logger.debug("Preparing to get activity data")

        # Convert to query parameters for the API request
        params = request.to_query_params()

        self.logger.debug("Getting activity data for domain: %s", request.domain_id)
        self.logger.debug("Query params: %s", params)

        response = self.client.request(
            method="GET", path=f"activity/{request.domain_id}", params=params
        )

        return self._create_response(response)

    def get_single(self, request: SingleActivityRequest) -> APIResponse:
        """
        Get a single activity by its ID.

        Args:
            request: A fully-validated SingleActivityRequest object

        Returns:
            APIResponse with single activity data
        """
        self.logger.debug("Preparing to get single activity")
        self.logger.debug("Getting single activity: %s", request.activity_id)

        response = self.client.request(
            method="GET", path=f"activities/{request.activity_id}"
        )

        return self._create_response(response)



"""Recipients API models for MailerSend SDK."""

from datetime import datetime
from typing import Optional, List, Union, Dict, Any
from pydantic import BaseModel, Field, field_validator


# Query Parameters Models
class RecipientsListQueryParams(BaseModel):
    """Query parameters for listing recipients."""

    domain_id: Optional[str] = None
    page: int = Field(default=1, ge=1)
    limit: int = Field(default=25, ge=10, le=100)

    @field_validator("domain_id")
    @classmethod
    def validate_domain_id(cls, v: Optional[str]) -> Optional[str]:
        """Validate and clean domain_id."""
        if v is not None:
            return v.strip()
        return v

    def to_query_params(self) -> Dict[str, Any]:
        """Convert to query parameters dictionary, excluding None values."""
        params = {}
        if self.domain_id is not None:
            params["domain_id"] = self.domain_id
        params["page"] = self.page
        params["limit"] = self.limit
        return params


class SuppressionListQueryParams(BaseModel):
    """Query parameters for listing suppression entries."""

    domain_id: Optional[str] = None
    page: int = Field(default=1, ge=1)
    limit: int = Field(default=25, ge=10, le=100)

    @field_validator("domain_id")
    @classmethod
    def validate_domain_id(cls, v: Optional[str]) -> Optional[str]:
        """Validate and clean domain_id."""
        if v is not None:
            return v.strip()
        return v

    def to_query_params(self) -> Dict[str, Any]:
        """Convert to query parameters dictionary, excluding None values."""
        params = {}
        if self.domain_id is not None:
            params["domain_id"] = self.domain_id
        params["page"] = self.page
        params["limit"] = self.limit
        return params


# Request Models
class RecipientsListRequest(BaseModel):
    """Request model for listing recipients."""

    query_params: RecipientsListQueryParams = Field(
        default_factory=RecipientsListQueryParams
    )

    def to_query_params(self) -> Dict[str, Any]:
        """Convert to query parameters dictionary."""
        return self.query_params.to_query_params()


class RecipientGetRequest(BaseModel):
    """Request model for getting a single recipient."""

    recipient_id: str

    @field_validator("recipient_id")
    @classmethod
    def validate_recipient_id(cls, v: str) -> str:
        """Validate and clean recipient_id."""
        if not v or not v.strip():
            raise ValueError("recipient_id cannot be empty")
        return v.strip()


class RecipientDeleteRequest(BaseModel):
    """Request model for deleting a recipient."""

    recipient_id: str

    @field_validator("recipient_id")
    @classmethod
    def validate_recipient_id(cls, v: str) -> str:
        """Validate and clean recipient_id."""
        if not v or not v.strip():
            raise ValueError("recipient_id cannot be empty")
        return v.strip()


class SuppressionListRequest(BaseModel):
    """Request model for listing suppression entries."""

    query_params: SuppressionListQueryParams = Field(
        default_factory=SuppressionListQueryParams
    )

    def to_query_params(self) -> Dict[str, Any]:
        """Convert to query parameters dictionary."""
        return self.query_params.to_query_params()


class SuppressionAddRequest(BaseModel):
    """Request model for adding recipients to suppression lists."""

    domain_id: str
    recipients: Optional[List[str]] = None
    patterns: Optional[List[str]] = None

    @field_validator("domain_id")
    @classmethod
    def validate_domain_id(cls, v: str) -> str:
        """Validate and clean domain_id."""
        if not v or not v.strip():
            raise ValueError("domain_id cannot be empty")
        return v.strip()

    @field_validator("recipients")
    @classmethod
    def validate_recipients(cls, v: Optional[List[str]]) -> Optional[List[str]]:
        """Validate recipients list."""
        if v is not None:
            if not v:
                raise ValueError("recipients list cannot be empty if provided")
            # Clean and validate each email
            cleaned = []
            for email in v:
                if not email or not email.strip():
                    raise ValueError("recipient email cannot be empty")
                cleaned.append(email.strip())
            return cleaned
        return v

    @field_validator("patterns")
    @classmethod
    def validate_patterns(cls, v: Optional[List[str]]) -> Optional[List[str]]:
        """Validate patterns list."""
        if v is not None:
            if not v:
                raise ValueError("patterns list cannot be empty if provided")
            # Clean each pattern
            cleaned = []
            for pattern in v:
                if not pattern or not pattern.strip():
                    raise ValueError("pattern cannot be empty")
                cleaned.append(pattern.strip())
            return cleaned
        return v


class SuppressionDeleteRequest(BaseModel):
    """Request model for deleting from suppression lists."""

    domain_id: Optional[str] = None
    ids: Optional[List[str]] = None
    all: Optional[bool] = None

    @field_validator("domain_id")
    @classmethod
    def validate_domain_id(cls, v: Optional[str]) -> Optional[str]:
        """Validate and clean domain_id."""
        if v is not None:
            return v.strip()
        return v

    @field_validator("ids")
    @classmethod
    def validate_ids(cls, v: Optional[List[str]]) -> Optional[List[str]]:
        """Validate ids list."""
        if v is not None:
            if not v:
                raise ValueError("ids list cannot be empty if provided")
            # Clean each id
            cleaned = []
            for id_val in v:
                if not id_val or not id_val.strip():
                    raise ValueError("id cannot be empty")
                cleaned.append(id_val.strip())
            return cleaned
        return v



"""Webhooks resource for MailerSend SDK."""

from .base import BaseResource
from ..models.base import APIResponse
from ..models.webhooks import (
    WebhooksListRequest,
    WebhookGetRequest,
    WebhookCreateRequest,
    WebhookUpdateRequest,
    WebhookDeleteRequest,
)


class Webhooks(BaseResource):
    """Webhooks API resource for managing webhooks."""

    def list_webhooks(self, request: WebhooksListRequest) -> APIResponse:
        """List webhooks for a domain.

        Args:
            request: The webhooks list request

        Returns:
            APIResponse with WebhooksListResponse data
        """
        self.logger.debug("Starting list_webhooks operation")
        self.logger.debug("Webhooks list request: %s", request)

        # Extract query parameters
        params = request.to_query_params()

        self.logger.debug("Listing webhooks with params: %s", params)

        # Make API call
        response = self.client.request(method="GET", path="webhooks", params=params)

        # Create standardized response
        return self._create_response(response)

    def get_webhook(self, request: WebhookGetRequest) -> APIResponse:
        """Get a single webhook by ID.

        Args:
            request: The webhook get request

        Returns:
            APIResponse with WebhookResponse data
        """
        self.logger.debug("Starting get_webhook operation")
        self.logger.debug("Webhook get request: %s", request)

        # Make API call
        response = self.client.request(
            method="GET", path=f"webhooks/{request.webhook_id}"
        )

        # Create standardized response
        return self._create_response(response)

    def create_webhook(self, request: WebhookCreateRequest) -> APIResponse:
        """Create a new webhook.

        Args:
            request: The webhook create request

        Returns:
            APIResponse with WebhookResponse data
        """
        self.logger.debug("Starting create_webhook operation")

        # Prepare request body
        data = request.model_dump(exclude_none=True)

        self.logger.debug("Creating webhook: %s", request.name)

        # Make API call
        response = self.client.request(method="POST", path="webhooks", body=data)

        # Create standardized response
        return self._create_response(response)

    def update_webhook(self, request: WebhookUpdateRequest) -> APIResponse:
        """Update an existing webhook.

        Args:
            request: The webhook update request

        Returns:
            APIResponse with WebhookResponse data
        """
        self.logger.debug("Starting update_webhook operation")
        self.logger.debug("Webhook update request: %s", request)

        # Prepare request body - exclude webhook_id (goes in URL) and None fields
        data = request.model_dump(exclude_none=True, exclude={"webhook_id"})

        self.logger.debug("Updating webhook: %s", data)

        # Make API call
        response = self.client.request(
            method="PUT", path=f"webhooks/{request.webhook_id}", body=data
        )

        # Create standardized response
        return self._create_response(response)

    def delete_webhook(self, request: WebhookDeleteRequest) -> APIResponse:
        """Delete a webhook.

        Args:
            request: The webhook delete request

        Returns:
            APIResponse with empty data
        """
        self.logger.debug("Starting delete_webhook operation")
        self.logger.debug("Webhook delete request: %s", request)

        # Make API call
        response = self.client.request(
            method="DELETE", path=f"webhooks/{request.webhook_id}"
        )

        # Create standardized response
        return self._create_response(response)



from typing import List, Optional, Union
from datetime import datetime

from ..models.activity import (
    ActivityRequest,
    ActivityQueryParams,
    SingleActivityRequest,
)


class ActivityBuilder:
    """Builder for constructing activity requests."""

    def __init__(self):
        self._domain_id: Optional[str] = None
        self._page: Optional[int] = None
        self._limit: Optional[int] = None
        self._date_from: Optional[Union[datetime, int]] = None
        self._date_to: Optional[Union[datetime, int]] = None
        self._event: List[str] = []

    def domain_id(self, domain_id: str) -> "ActivityBuilder":
        """Set the domain ID."""
        self._domain_id = domain_id
        return self

    def page(self, page: int) -> "ActivityBuilder":
        """Set the page number."""
        self._page = page
        return self

    def limit(self, limit: int) -> "ActivityBuilder":
        """Set the limit per page."""
        self._limit = limit
        return self

    def date_from(self, date_from: Union[datetime, int]) -> "ActivityBuilder":
        """Set the start date."""
        self._date_from = date_from
        return self

    def date_to(self, date_to: Union[datetime, int]) -> "ActivityBuilder":
        """Set the end date."""
        self._date_to = date_to
        return self

    def event(self, event: str) -> "ActivityBuilder":
        """Add an event type to filter by."""
        if event not in self._event:
            self._event.append(event)
        return self

    def events(self, events: List[str]) -> "ActivityBuilder":
        """Set multiple event types to filter by."""
        self._event = list(events)
        return self

    def copy(self) -> "ActivityBuilder":
        """Create a copy of this builder."""
        builder = ActivityBuilder()
        builder._domain_id = self._domain_id
        builder._page = self._page
        builder._limit = self._limit
        builder._date_from = self._date_from
        builder._date_to = self._date_to
        builder._event = self._event.copy()
        return builder

    def reset(self) -> "ActivityBuilder":
        """Reset all parameters."""
        self._domain_id = None
        self._page = None
        self._limit = None
        self._date_from = None
        self._date_to = None
        self._event = []
        return self

    def _convert_to_timestamp(self, value: Union[datetime, int]) -> int:
        """Convert datetime to timestamp if needed."""
        if isinstance(value, datetime):
            return int(value.timestamp())
        return value

    def build_list_request(self) -> ActivityRequest:
        """Build the ActivityRequest object for listing activities."""
        return self.build()
    
    def build(self) -> ActivityRequest:
        """Build the ActivityRequest object."""
        # Convert dates to timestamps if needed
        date_from = None
        if self._date_from is not None:
            date_from = self._convert_to_timestamp(self._date_from)

        date_to = None
        if self._date_to is not None:
            date_to = self._convert_to_timestamp(self._date_to)

        # Build the query parameters
        query_params = ActivityQueryParams(
            page=self._page,
            limit=self._limit,
            date_from=date_from,
            date_to=date_to,
            event=self._event if self._event else None,
        )

        # Build the complete request
        return ActivityRequest(domain_id=self._domain_id, query_params=query_params)


class SingleActivityBuilder:
    """
    Builder for constructing SingleActivityRequest objects.

    Provides a fluent interface for building requests to get a single activity.
    """

    def __init__(self):
        """Initialize the builder with default values."""
        self._activity_id: Optional[str] = None

    def activity_id(self, activity_id: str) -> "SingleActivityBuilder":
        """
        Set the activity ID.

        Args:
            activity_id: The unique identifier of the activity

        Returns:
            Self for method chaining
        """
        self._activity_id = activity_id
        return self

    def copy(self) -> "SingleActivityBuilder":
        """
        Create a copy of this builder.

        Returns:
            A new SingleActivityBuilder instance with the same configuration
        """
        new_builder = SingleActivityBuilder()
        new_builder._activity_id = self._activity_id
        return new_builder

    def reset(self) -> "SingleActivityBuilder":
        """
        Reset the builder to its initial state.

        Returns:
            Self for method chaining
        """
        self._activity_id = None
        return self

    def build_get_request(self) -> SingleActivityRequest:
        """Build the SingleActivityRequest object for getting a single activity."""
        return self.build()
    
    def build(self) -> SingleActivityRequest:
        """
        Build the SingleActivityRequest object.

        Returns:
            A configured SingleActivityRequest instance

        Raises:
            ValueError: If required fields are missing
        """
        if not self._activity_id:
            raise ValueError("activity_id is required")

        return SingleActivityRequest(activity_id=self._activity_id)
